* David,
* This is what I think is the status of the program:
* The control structure for ENSDF -> AGS (13 or 23) and
* AGS -> ENSDF (31 or 32) are in place. The 13,23 conversion
* works, but the format is LVL. That means that you should
* start by modifying the routine WAGS. The 31,32 conversion
* does something, but I do not know what. Anyway, when you 
* get that far, you should rewrite RAGS to read the AGS format.

C ***************************************************************************
C                                                             FCONV.FOR
C     ************************
C     PROGRAM FormatConversion
C     ************************
C
C     Purpose
C     *******
C     Converts nuclear level scheme data between different formats:
C
C       READS:
C          ENSDF (Evaluated Nuclear Structure Data File)
C          ENSDF/2 (ENSDF with level and band indices (LBL))
C          ESCL8R .AGS format (David Radford, Oak Ridge)
C
C       WRITES:
C          ENSDF
C          ENSDF/2 (ENSDF with level and band indices)
C          Radware .AGS format
C
C       CHECKS (errors):
C          Level order
C          Transition energy consistent with difference in level energies
C          Multipolarity consistent with level spins
C          Two multipolarities given if mixing ratio is given
C          On writing AGS format: check bands for inconsistent
C          parity or spin (e.g. two levels with the same spin)
C
C     Programmers
C     ***********
C     ENSDF read and ENSDF/2 write are adapted from the program FLEVEL
C       by Richard B Firestone (LBL)
C     Main program, data interface, ENSDF write and ENSDF/2 read,
C       Radware .AGS format
C       by Peter Ekstrom (Lund)
C
C     E-mail addresses:
C       Peter Ekstrom        Peter.Ekstrom@Nuclear.LU.Se
C       Richard Firestone    RBF@LBL.Gov
c       David Radford        RadfordD@mail.phy.ornl.gov
C
C ***************************************************************************
C
C     Files required
C     **************
C     FCONV.FOR        Main control program and read/write routines
C     fconv.inc        Include file (common data blocks and parameter
C                      definitions)
C     open.inc         Open unit 2 include file
C     STR77.FOR        Routines mainly to decode/encode ENSDF
C                      character string data
C
C     Compiling/linking
C     *****************
C     * Make sure the open statements in file open.inc are correct and
C       that the values of the parameter definitions (array bounds) in
C       fconv.inc are big enough. The program does check that these
C       bounds are not exceeded
C     * Compile STR77 and FCONV
C     * Link FCONV with STR77
C
C     With the exeption of the VAX Fortran extension CARRIAGECONTROL='LIST'
C     in the file open statements the program adheres strictly to
C     Fortran 77 standard (ah, and the DO - ENDDO construct which is
C     almost standard).
C
C ***************************************************************************
C
C     Copyright notice/errors
C     ***********************
C     This program may be freely distributed and copied. If you add
C     new formats, please send the modified program to Peter Ekstrom,
C     so that your format may be included in future versions of
C     the program.
C
C     Please report 'undocumented features' to Peter Ekstrom. Please
C     describe exactly what goes wrong under which circumstances.
C     If possible, supply input data that reproduce the problem.
C
C ***************************************************************************
C
C     File naming conventions
C     ***********************
C     ENSDF file for mass nnn: nnn.ENS e.g. 090.ENS for mass 90
C     ENSDF/2 file for mass nnn: nnn.ENX e.g. 090.ENX for mass 90
C     Output ENSDF file for mass nnn: nnn.EN
C     The log file is called FCONV.LOG and is created in the
C       current directory.
C     For some modes a file with file list is required.
C
C ****************************************************************************
C
C     Input (keyboard)
C     ****************
C     1.  Mode, Logon
C         Mode:
C         1 Automatic ENSDF->ENSDF2 conversion (nnn.ENS -> nnn.ENX)
C         2 Automatic ENSDF2->ENSDF conversion (nnn.ENX -> nnn.EN)
C         3 ENSDF->ENSDF2 conversion
C         4 ENSDF2->ENSDF conversion
C            Options 1-4 retain all information (all cards) of an
C            ENSDF or ENSDF/2 file.
C         5 ENSDF2->Band list (file list file)
C         6 Not used                  
C         7 ENSDF2->AGS format (A-chain conversion) Not implemented
C         8 ENSDF2->AGS format (file list file) Not implemented
C         The following options (where the first digit is the
C            read format and second digit is the write format)
C            only retains data that are saved in the common
C            blocks (see fconv.inc):
C            Format codes:
C            1. ENSDF
C            2. ENSDF/2
C            3. Radford's .AGS format
C            e.g.
C         12 ENSDF->ENSDF2 using common block data (EtoE2 does not write)
C         21 ENSDF2->ENSDF using common block data (E2toE does not write)
C         32 AGS->ENSDF2 format 
C         Logon: =0, errors printed to FCONV.LOG
C                =1, messages/errors printed to FCONV.LOG
C                =2, full printout to FCONV.LOG (for debugging)
C     2.  Input directory (current=blank)
C     3.  Output directory (current=blank)
C     4.  Data set selection string (e.g. DECAY, ADOPTED. Blank for
C         all data sets)
C     5.  For automatic ENSDF-ENSDF2 conversion: enter upper
C         and lower limit on mass number (missing files OK)
C         For manual conversion: enter input file name
C                                enter output file name
C         For manual conversion: Repeat 5 ad infinitum (blank to stop)
C
C     Input file: unit 1
C     Output file: unit 2
C     Log file: unit 4 (FCONV.LOG) contains a log of the conversion
C               procedures and errors encountered
C
C ****************************************************************************
C
c  Definition of ENSDF/2 format. For details on ENSDF format, see
c   Evaluated Nuclear Structure Data File, A Manual for Preparation
c   of Data Sets, J.K. Tuli, BNL-NCS-51655-Rev.87
c
c          1         2         3         4         5         6         7         8
c 12345678901234567890123456789012345678901234567890123456789012345678901234567890
c
c     Data set identification card:
c 159EU   *159SM B- DECAY                86WIZU,86MA12            88NDS     880825
c    1-5     nucid
c    9       *
c    10-39   data set identification
c    40-64   data set references
c    65-74   NDS reference
c    75-80   date of entry into ENSDF (yymmdd)
c
c     Information card (all numbers are right justified in given fields):
c  159 63 I     7        14         0         0         1159SM              0
c    1-3     a, mass number
c    4-6     z, atomic number
c    8       I
c    11-14   nlev, number of levels
c    21-24   ngam, number of gammas
c    31-34   nalpha, number of alphas
c    41-44   nbeta, number of betas
c    51-54   npar, number of parents
c    55-59   cpar(1), parent 1
c    60-64   cpar(2), parent 2
c    71-74   nband, number of bands
c
c     Band description card:
c   A   CL BAND(A)     5/2(413)    A=10.6,  B=0.004
c    3       band tag
c    7-8     CL
c    10-     band comment
c
c     Level card:
c     1  L 0.0         (5/2+)            18.1 M    1
c     1      Q or ? (from col. 80)
c     2      isomer tag (M)
c     3      band tag
c     4-5    level index (level number)
c     8      L
c     10     energy
c
c     Gamma card:
c  BA 2  G 179.2      1 60      6
c     1      Q or ? (from col. 80)
c     2      initial level band tag
c     3      final level band tag
c     4-5    final level index
c     8      G
c     10     gamma-ray energy
c
c 12345678901234567890123456789012345678901234567890123456789012345678901234567890
c          1         2         3         4         5         6         7         8
C
C ***************************************************************************
C
C  Hints and comments
C  ******************
C
C  FConv.log
C
C  Always scan the log file (FConv.log) for errors and warnings. Warnings
C  (marked *W*) can usually be neglected, but errors (marked *E*) have
C  to be corrected, or the output may be incorrect.
C
C
C  Radware AGS format
C
C  The mass number A is (deliberately) not defined in the AGS
C  format. Since A is needed in the NUCID field in the ENSDF format,
C  proceed as follows for AGS->ENSDF conversion:
C  1) Make sure that there is data for only one nuclide in the file
C  2) Add the following header as a comment among the first five lines:
C  ** AAAZZ 
C  e.g. 
C  ** 152Dy  
C
C  Note that is is NOT allowed to use AGS files
C  with data from more than one nuclide. The reason
C  for this is the structure of the ENSDF format. The only check for
C  this is that FConv gives an error if you mix integer and half-integer
C  spins in a file.
C
C  Unknown spins in AGS format
C  Non-band levels: spin=-1.0 or -0.5, spin label is -1 or -0
C  Band levels: band head spin -1 or -0.5, spin label -1 or -0
C               other levels 1.0,3.0... or 0.0,1.0... for even A
C                            1.5,3.5...    0.5,1.5... for odd A
C               unless given spins allow an extrapolation.
C  Spin sequence J+x: Treated as band levels with unknown band head spin.
C  If a value of J is given, it is used for a guess.
C
C ***************************************************************************
C
C  Revision log
C  ************
C
C    01/06/94/lpe (1.0):
C       Added default input and output directory
C
C    19/06/94/lpe (1.1):
C       SP+ and SN+ will be correctly interpreted if SN/SP are given
C         on a Q card
C       Added mode 5 (Band listing)
C       Fixed BAND card processing so that several tags are read
C         (e.g. BAND(A,B))
C       Added data set selection through string DSStr
C
C    23/06/94/lpe (1.1):
C       Check that mode is valid at input
C       Band + gammas list (mode 6)
C
C    03/07/94/lpe (1.2):
C       Preliminary version of routines to write Radford's
C         ESCL8R .LVL format
C
C    21/07/94/lpe (2.0):
C       Corrected LVL write
C       Written LVL read (preliminary)
C       Sorting of levels in energy order
C       Added reading E2 gamma cascade to RLVL
C       Added comment record for data transfer between formats
C       Removed some unused routines in STR77
C
C    26/07/94/lpe (2.0):
C       Corrected reading problems in RLVL
C       Retain FLAG= card also for band flag (card may contain more info)
C       No output - do not ask for output filename
C
C    10/07/95/lpe (2.0):
C       Allowed band comment as cL BAND....
C       Checked that Frank's correction for the BAND(c,d) construct
C         was taken care of
C       Added option 7: automatic ENSDF2->LVL conversion
C       Allow level out of order in E2toE if not write
C       Some format changes in WLVL
C       Made sure no warning messages (*W*) are printed unless
C         Logon>0
C    
C    13/07/95/lpe (2.0):
C       For mode=7: do not write lvl file without levels
C       Write number of levels in input and LVL file to LVL file
C
C    17/07/95/lpe (2.0):
C       Multipolarity error message only when BOTH L:s are inconsistent
C         with spins or when parity is wrong 
C
C    18/07/95/lpe (2.0)
C       Making sure that the case if one incorrect multipolarity
C         gives an error message
C
C    20/07/95/lpe (2.0)
C       Corrected problem in multipoarity checking when only one
C         multipolarity is given
C
C    08/08/95/lpe (2.0)
C       Skip levels for IOUT=0 instead of aborting for incorrect Sn/Sp
C
C    08/26/95/lpe (2.0)
C       Modified option 5 - band listing. More compact listing and it
C         lists only level energies and spins. Introduced file list.
C         Introduced spin modifier (e.g. J+20) and flags (?) for 
C         uncertain levels and transitions in common blocks. Gammas in
C         LVL format marked with *F 0 (firm) and *F 1 (tentative)
C         Levels in LVL format marked with *F 0 (firm) and 
C         *F 7 (tentative). Modified PutJPi to add spin modifier J.
C         Save JPi field of all band-heads. Print in band listing
C         format and in LVL format.
C
C    14/09/95/lpe (2.1)
C       Revised LVL format to support uncertain spin. All levels
C         converted to LVL format. Multipolarity not
C         printed in LVL format. Intensities of gammas renormalized
C         to 1 for ADOPTED LEVELS.
C       Uniqueness of band labels checked.
C       Consistent treatment of unknown spins and spin labels.
C       Correct J spacing for SD band.
C       List calculated band properties in WBand.
C    
C    21/09/95/lpe (2.1)
c       If there is a level comment with the same tag as a band,
c        write a FLAG= card
C
C    04/11/95/lpe (2.1)
C       Fixed bug that crashed program when unplaced gammas depopulated
C       states with E=0. This now causes an error message (*E*)
C
C    27/11/95/lpe (2.2)
C       Added mode=8 (ENSDF2 to LVL conversion with file list)
C       Badly fitting gammas only give *E* when difference is GT 1 keV.
C
C    21/02/98/lpe (3.0)
C       Fixed some problems with common blocks. The program
C       now works both on MS Powerstation and under VMS.
C
C    27/04/98/lpe (3.0)
C	  Added dummy mode 31. The program now converts to and from
C       AGS format, but the conversion routines ()
C	  need to be modified (they are actually LVL format).
C
C    07/05/98/dcr (3.0)
C       Extracted subroutine wags to a separate file, and modifed it 
C       extensively to write .ags files properly.  Delta-I = 1 bands
C       in ENSDF are split into two Delta-I = 2 bands in .ags.  Levels
C       without band assignments in ENSDF are put into one of 9 possible
C       bands in .ags, depending on parity (odd,even,unknown) and spin
C       (odd,even,unknown).  Comments and errors are written to a .txt
C       file (unit 22).  The main routine was modified to change the
C       filenames used, and to open/close the .txt file.  RAGS is still
C       wrong, so conversion from .ags to ENSDF does not work.

C ****************************************************************************
C
C

      implicit none

      character*70 inname,outname,form(0:3)*11,indir,outdir,listname
      character*10 outSuff
      character ext*3,outext*3
      character*20 dattim

      integer i,amin,amax,ii,mode,mode1,mode2,indirl,outdirl,outSuffl
      integer lenstr,j

      INCLUDE 'fconv.inc'

c$$
      data verstr /'FConv v3.1 98-10-15/lpe/dcr '/
      data form /'NONE','ENSDF','ENSDF/2','Radware AGS'/

      LComm=0
c     Define normal order
      do i=1,mlev
         lo(i)=i
         loo(i)=i
      enddo

C  Unit 4: Log file
      OPEN(UNIT=4,FILE='fconv.log',STATUS='UNKNOWN')


c  Input Mode and Logon
      write(*,*) verstr
120   write(*,*)'Mode:'
      write(*,*)' 1 Automatic ENSDF->ENSDF2 conversion ',
     $ '(nnn.ENS->nnn.ENX)'
      write(*,*)' 2 Automatic ENSDF2->ENSDF conversion ',
     $ '(nnn.ENX->nnn.EN)'
      write(*,*)' 3 ENSDF->ENSDF2 conversion'
      write(*,*)' 4 ENSDF2->ENSDF conversion'
      write(*,*)' 5 ENSDF2->Band list (file list)'
      write(*,*)' 6 Not used'
      write(*,*)' 7 Automatic ENSDF2->AGS conversion'
      write(*,*)' 8 Automatic ENSDF2->AGS conversion (file list)'
      write(*,*)'   Conversion codes:'
      write(*,*)'   0 No output file'
      write(*,*)'   1 ENSDF, 2 ENSDF/2, 3 ESCL8R AGS'
      write(*,*)' 12 ENSDF->ENSDF2 using common block data'
      write(*,*)' 21 ENSDF2->ENSDF using common block data'
	write(*,*)' 13 ENSDF->AGS using common block data'
	write(*,*)' 31 AGS->ENSDF'
      write(*,*)'Logon =0, errors printed to FCONV.LOG'
      write(*,*)'Logon =1, messages/errors printed to FCONV.LOG'
      write(*,*)'Logon =2, full printout to FCONV.LOG (for debugging)'
      write(*,*) ' Enter Mode, Logon:'
      read(*,*) mode,logon

      mode1=mode/10
      mode2=mode-10*mode1


c     Make sure supported conversion mode is chosen
      if(mode.eq.1.or.mode.eq.2.or.mode.eq.3.or.mode.eq.4) goto 121
      if(mode.eq.5.or.mode.eq.5.or.mode.eq.7.or.mode.eq.8) goto 121

      if(mode1.gt.3.or.mode2.gt.3) then
         write(*,*)'**** Invalid mode, try again ****'
         goto 120
      endif
121   continue

      write(4,*) verstr
      if(mode.ge.10) then
         write(4,*)' ',form(mode1),' -> ',form(mode2),' conversion'
      else
         write(4,*)' MODE= ',mode
      endif
      write(4,*)' LOGON=',logon
c------------------------------
c      call date(sdate)
c      call time(stime)
      call datetime(dattim)
      sdate = dattim(1:9)
      stime = dattim(11:18)
c------------------------------
      write(4,*) ' Date/time: ',sdate,' ',stime
      write(4,*)
      write(4,*)'   Conventions for message classification:'
      write(4,*)'     #   Information on automatic decisions made'
      write(4,*)'          by the program (only if LOGON>=1).'
      write(4,*)'     &   Previous problem resolved (only if LOGON>=1)'
      write(4,*)'     *E* Errors '
      write(4,*)'     *W* Warnings (only of LOGON>=1)'
      write(4,*)

c  Input Input and Output directories
      write(*,*)' Enter Input directory (Return for current directory)'
      read(*,'(A)') indir
      write(*,*)' Enter Output directory (Return for current directory)'
      read(*,'(A)') outdir
      call tbsup(indir)
      call tbsup(outdir)
      indirl=lenstr(indir)
      outdirl=lenstr(outdir)
      write(4,*) 'Input directory: ',indir
      write(4,*) 'Output directory: ',outdir

c     Input data set selection string

      write(*,*) ' Enter data set selection string (Case is '
      write(*,*) ' significant, CR for all data sets):'
      read(*,'(A)') DSStr
      call tbsup(DSStr)
      DSStrL=lenstr(DSStr)
	if(DSStrL.gt.0) then
		write(*,*) 'Data set selection string:'
		write(*,*) '"'//DSStr(1:DSStrL)//'"'
		write(4,*) 'Data set selection string:'
		write(4,*) '"'//DSStr(1:DSStrL)//'"'
	endif

c  For automatic conversion, input A limits
      if(mode.eq.1.or.mode.eq.2.or.mode.eq.7) then
         write(*,*) ' Enter A(min), A(max)'
         read(*,*) amin,amax
      endif

	if(logon.ge.2) call commonBlocks(2,'After data input')

c  Automatic ENSDF->ENSDF2 conversion
      if(mode.eq.1) then
         do 10 i=amin,amax
            write(inname,'(I3.3,A)') i,'.ENS'
            write(outname,'(I3.3,A)') i,'.ENX'
C           Unit 1: Input file
           if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD',ERR=2)
           else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD',ERR=2)
           endif

            goto 3
2           continue
            write(*,*)' Input file for A=',i,' not found'
            goto 10

3           continue

      include 'open.inc'

            TheEnd=.false.
c           Convert one data set at a time
            do ii=1,500
               CALL EtoE2(1,2)
               call CheckDS
               if(NewDS.and.logon.ge.2) call pridat
               if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
               if(TheEnd) goto 5
            enddo

5        continue
         close(1)
         close(2)
         close(22)
10       continue

c  Automatic ENSDF2->ENSDF conversion
      elseif(mode.eq.2) then
         do 20 i=amin,amax
            write(inname,'(I3.3,A)') i,'.ENX'
            write(outname,'(I3.3,A)') i,'.EN'
C           Unit 1: Input file
           if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD',
     $           ERR=12)
           else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD',
     $           ERR=12)
           endif
            goto 13
12          continue
            write(*,*)' Input file for A=',i,' not found'
            goto 20

13          continue

      include 'open.inc'

            TheEnd=.false.
c           Convert one data set at a time
            do ii=1,500
               CALL E2toE(1,2)
               call CheckDS
               if(NewDS.and.logon.ge.2) call pridat
               if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
               if(TheEnd) goto 15
            enddo

15        continue
         close(1)
         close(2)
         close(22)
20       continue


c  Automatic ENSDF2->AGS conversion
      elseif(mode.eq.7) then
         do 220 i=amin,amax
            write(inname,'(I3.3,A)') i,'.ENX'
C           Unit 1: Input file
           if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD',
     $           ERR=112)
           else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD',
     $           ERR=112)
           endif
            goto 113
112         continue
            write(*,*)' Input file for A=',i,' not found'
            goto 220

113         continue


            TheEnd=.false.
c           Convert one data set at a time
            do ii=1,500
               CALL E2toE(1,0)

c              Write data
               CLOSE (2)
               if(NLev.le.0) goto 114

               if(nucid(5:5).eq.' ') then
                  write(outname,'(I3.3,A,A)') i,NUCID(4:4),'.AGS'
               else
                  write(outname,'(I3.3,A,A)') i,NUCID(4:5),'.AGS'
               endif
               include 'open.inc'
               if(NewDS) call WAGS(2,mode)
               call CheckDS
               if(NewDS.and.logon.ge.2) call pridat
               if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
               close(2)
               close(22)
               if(TheEnd) goto 115
114            continue
            enddo

115      continue
         close(1)
220      continue


c  Manual conversion
      elseif(mode.eq.3.or.mode.eq.4) then
30       continue

         if(mode.eq.3) write(*,*)' ENSDF -> ENSDF/2'
         if(mode.eq.4) write(*,*)' ENSDF/2 -> ENSDF'

         write(*,*) ' Input filename (Blank=STOP):'
         read(*,'(A)') inname
         if(inname.eq.' ') goto 99
         write(*,*) ' Output filename'
         read(*,'(A)') outname

C        Unit 1: Input file
         if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD')
         else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD')
         endif
         include 'open.inc'

         TheEnd=.FALSE.

         do i=1,500
            if(mode.eq.3) then
               CALL EtoE2(1,2)
            elseif(mode.eq.4) then
               CALL E2toE(1,2)
            endif
            call CheckDS
            if(NewDS.and.logon.ge.2) call pridat
            if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
            if(TheEnd) goto 40
         enddo
40       continue
         close(1)
         close(2)
         close(22)
         goto 30

c  Band list
      elseif(mode.eq.5) then
130      continue

         if(mode.eq.5) write(*,*)' ENSDF/2 -> Band list'

         write(*,*) ' Name of list file'
         read(*,'(A)') listname

         if(indirl.gt.0) then
            OPEN(UNIT=10,FILE=indir(1:indirl)//listname,STATUS='OLD')
         else
            OPEN(UNIT=10,FILE=listname,STATUS='OLD')
         endif

         write(*,*) ' Output filename:'
         read(*,'(A)') outname

         ext=' '
         include 'open.inc'

131      continue
         call nextfile(10,inname,ext,dumstr,' ')
         if(inname.eq.' ') goto 140

C        Unit 1: Input file

         if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD')
         else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD')
         endif

         TheEnd=.FALSE.

         do i=1,500
            CALL E2toE(1,0)
            call CheckDS
            call WBAND(2)
            if(NewDS.and.logon.ge.2) call pridat
            if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
            if(TheEnd) then
               close(1)
               goto 131
            endif
         enddo
140      continue
         close(1)
         close(2)
         close(22)
         stop    

      elseif(mode.eq.8) then
150      continue

         write(*,*)' ENSDF/2 -> AGS format (file list)'

         write(*,*) ' Name of list file'
         read(*,'(A)') listname

         if(indirl.gt.0) then
            OPEN(UNIT=10,FILE=indir(1:indirl)//listname,STATUS='OLD')
         else
            OPEN(UNIT=10,FILE=listname,STATUS='OLD')
         endif

         write(*,*) ' Output filename extension:'
         read(*,'(A)') outext
         ext='   '

151      continue
         call nextfile(10,inname,ext,outname,outext)
         if(inname.eq.' ') goto 160

C        Unit 1: Input file
         if(indirl.gt.0) then         
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD')
         else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD')
         endif
c        UNIT 2: Output file
         include 'open.inc'


         TheEnd=.FALSE.

         do i=1,2000
            CALL E2toE(1,0)
            call CheckDS
            if(NLev.gt.0) then
               call WAGS(2,mode)
               if(NewDS.and.logon.ge.2) call pridat
               if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
            endif
            if(TheEnd) then
               close(1)
               close(2)
               close(22)
               goto 151
            endif
         enddo
160      continue
         close(1)
         close(2)
         close(22)
         stop    

c  Manual ENSDF <-> ENSDF2 conversion using common block data
      elseif(mode.ge.10.and.mode1.le.2.and.mode2.le.2) then
50       continue

         write(*,*) form(mode1),' -> ',form(mode2),' conversion'

         write(*,*) ' Input filename (Blank=STOP):'
         read(*,'(A)') inname
         if(inname.eq.' ') goto 99
         if(mode2.gt.0) then
            write(*,*) ' Output filename:'
            read(*,'(A)') outname
         endif

C        Unit 1: Input file
         if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD')
         else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD')
         endif

      include 'open.inc'

         TheEnd=.FALSE.

         do i=1,500
c           Read data
            if(mode1.eq.1) CALL EtoE2(1,0)
            if(mode1.eq.2) CALL E2toE(1,0)
            if(mode1.eq.3) CALL RAGS(inname)

c           Write data
            if(NewDS.and.mode2.eq.1) call WENSDF(2)
            if(NewDS.and.mode2.eq.2) call WENSDF2(2)
            if(NewDS.and.mode2.eq.3) call WAGS(2,mode)
c           Print data to log file

c           Check data
            call CheckDS
            if(NewDS.and.logon.ge.2) call pridat
c           Show progress on screen
            if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
            if(TheEnd) goto 60
         enddo
60       continue
         close(1)
         close(2)
         close(22)
         goto 50

c  Manual ENSDF/ENSDF2 -> AGS conversion using common block data
      elseif(mode.ge.10.and.(mode1.le.2.and.mode2.eq.3)) then
51       continue

         write(*,*) form(mode1),' -> ',form(mode2),' conversion'

         write(*,*) ' Input filename (Blank=STOP):'
         read(*,'(A)') inname
         if(inname.eq.' ') goto 99
         if(mode2.gt.0) then
            write(*,*) ' Suffix for output filename:'
            read(*,'(A)') outSuff
		  call tbsup(outSuff)
		  outSuffl=lenstr(outSuff)
         endif

C        Unit 1: Input file
         if(indirl.gt.0) then
            OPEN(UNIT=1,FILE=indir(1:indirl)//inname,STATUS='OLD')
         else
            OPEN(UNIT=1,FILE=inname,STATUS='OLD')
         endif

         TheEnd=.FALSE.

         do i=1,2000
c           Read data
            if(mode1.eq.1) CALL EtoE2(1,0)
            if(mode1.eq.2) CALL E2toE(1,0)
            if(NLev.ge.1) then
 
c              Write data
               ii = 3
               do while (ii.lt.5 .and.
     +                   NUCID(ii+1:ii+1).ne.' ' .and.
     +                   NUCID(ii+1:ii+1).ne.char(0))
                   ii = ii+1
               enddo
               if(outSuffl.gt.0) then
                  outname = NUCID(1:3)//'/'//NUCID(1:ii)//'_ensdf_'
     +                                 //outSuff(1:outSuffl)//'.ags'
               else
                  outname = NUCID(1:ii)//'_ensdf.ags'
               endif
               if (outname(1:2).eq.'  ') then
                  outname(1:2) = '00'
                  outname(5:) = outname(7:)
               elseif (outname(1:1).eq.' ') then
                  outname(1:1) = '0'
                  outname(5:) = outname(6:)
               endif
               include 'open.inc'

               if(NewDS.and.mode2.eq.3) call WAGS(2,mode)
c              Print data to log file

c              Check data
               call CheckDS
               if(NewDS.and.logon.ge.2) call pridat
c              Show progress on screen
               if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
               if(TheEnd) goto 61
            endif
         enddo
61       continue
         close(1)
         close(2)
         close(22)
         goto 51

c  Manual AGS -> ENSDF/ENSDF2 conversion using common block data
      elseif(mode.ge.10.and.mode1.eq.3.and.mode2.le.2) then
52       continue

         write(*,*) form(mode1),' -> ',form(mode2),' conversion'

         write(*,*) ' Input filename (Blank=STOP):'
         read(*,'(A)') inname
         if(inname.eq.' ') goto 99
         if(mode2.gt.0) then
            write(*,*) ' Output filename:'
            read(*,'(A)') outname
         endif

C        Unit 1: Input file
         if(indirl.gt.0) then
            OPEN(UNIT=25,FILE=indir(1:indirl)//inname,STATUS='OLD')
         else
            OPEN(UNIT=25,FILE=inname,STATUS='OLD')
         endif

      include 'open.inc'

         TheEnd=.FALSE.

         do i=1,500
c           Read data
            if(mode1.eq.1) CALL EtoE2(1,0)
            if(mode1.eq.2) CALL E2toE(1,0)
            if(mode1.eq.3) CALL RAGS(inname)

c           Write data
            if(NewDS.and.mode2.eq.1) call WENSDF(2)
            if(NewDS.and.mode2.eq.2) call WENSDF2(2)
            if(NewDS.and.mode2.eq.3) call WAGS(2,mode)
c           Print data to log file

c           Check data
            call CheckDS
            if(NewDS.and.logon.ge.2) call pridat
c           Show progress on screen
            if(NewDS) write(*,*) NUCID,'  ',DSID(1:71)
            if(TheEnd) goto 62
         enddo
62       continue
         close(1)
         close(2)
         close(22)
         goto 52


      endif

99    continue
      STOP
      END

      SUBROUTINE PRIDAT

      INCLUDE 'fconv.inc'
      character*(2) dum

c     Write header
      write(4,102) NUCID,A,Z,IODD
102   format (//' New data set: Nucid= ',A,'  A=',I3,'  Z=',
     $          I3,'  IODD=',I2)

      if(Sep) then
         write(4,*) SNucId,' Sn=',Sn,DSn
         write(4,*) SNucId,' Sp=',Sp,DSp
         write(4,*) '  KRatio=',KRatio
      endif
      write(4,*)

      write(4,'('' Bands:'')')
      do i=1,NBAND
         write(4,'(1X,I3,1X,A,1X,A)') i,BD(i),BANDID(i)
         write(4,'(''     Band head spin:'',A)') BdStr(i)
      enddo

      write(4,'('' Levels/gammas:'')')
      do i=1,NLev
         ii=loo(i)
         call charad(lo(ii),dum)
         if(ii.gt.0) write(4,100)
     $        ii,el(ii),del(ii),UncLev(ii),els(ii),l(ii),ndec(ii),
     $        mel(ii),Ban(ii),spin(ii),parity(ii),jpimod(ii),JOffs(ii),
     $        t(ii),dt(ii),ts(ii),dum
100      format(I4,F9.2,F7.2,A,A,I5,1X,I3,
     $          1X,A,1X,A,1X,I3,A,A,A,
     $          1X,G10.4,1X,G10.4,A,2X,A)

         do j=1,ngam
            if(inlev(j).eq.ii)
     $      write(4,101) egam(j),degam(j),
     $      UncTr(j),egams(j),igam(j),digam(j),igams(j),
     $      inlev(j),finlev(j),
     $      Ban(inlev(j)),Ban(finlev(j)),
     $      ML(1,j),ML(2,j),MLPar(j),MLMod(j),
     $      mult(j),dsign(j),d(j),dd(j),dd2(j),ds(j)
101         format(4X,F9.2,F5.2,A,A,2X,F10.3,F8.3,A,2I4,
     $             1X,A,1X,A,
     $             1X,2I3,1X,A,1X,A,
     $             1X,A,1X,A,F8.3,2F6.3,A)
         enddo

      enddo

      return
      END

      SUBROUTINE CheckDS

      implicit none
      INCLUDE 'fconv.inc'
      integer i,ii,iii,iiold,j,jj,jjj,prev,next,i1
      double precision eold

      iiold=0
      eold=0

      do i=1,ngam
         ii=inlev(i)
         iii=finlev(i)
C        Determine previous/next final level
         prev=0
         next=NLev
         if(iii.gt.1) prev=iii-1
         if(iii.lt.NLev) next=iii+1
         i1=i+1
         do j=i1,ngam
            jj=inlev(j)
            jjj=finlev(j)

c           Check for doubly matched gammas
            if(ii.eq.jj.and.iii.eq.jjj) then
               write(4,100)'*E*',ii,mel(ii),Ban(ii),el(ii),iii,mel(iii),
     $                      Ban(iii),el(iii),egam(i),egam(j)
100            format(/1X,A' Doubly matched gammas from level',
     $          I5,1X,A,1X,A,F8.2,' to level',I5,1X,A,1X,A,F8.2,
     $          /2X,F10.2,F10.2)
            write(4,103) el(prev),del(prev),el(next),del(next)
            endif
         enddo

c        Check that gamma fits between levels
         call chgam(el(ii),del(ii),el(iii),del(iii),egam(i),degam(i),A,
     $       el(prev),del(prev),el(next),del(next),logon)

c        Check gamma multipolarity against spin/parity
         call chmult(Spin(ii), Parity(ii), JPiMod(ii),
     $               Spin(iii),Parity(iii),JPiMod(iii),
     $               ML(1,i),  MLPar(i),   MLMod(i),mult(i),Iodd,
     $               egam(i),el(ii),el(iii),d(i),dd(i),dd2(i),
     $               el(prev),del(prev),el(next),del(next),logon)

c        Check gamma ordering
         if(egam(i).lt.eold.and.ii.eq.iiold) then
            if(logon.gt.0) write(4,101) '*W*',el(ii),eold,egam(i)
101         format(/1X,A,' Gammas in the wrong order, level=',F10.2,
     $             '. Gammas:',2F10.2)

         endif
         iiold=ii
         eold=egam(i)

c        Make sure levels have the same MEL letter
         if(MEL(ii).ne.MEL(iii).and.logon.gt.0) then
            write(4,201) '*W*',egam(i),mel(ii),el(ii),mel(iii),el(iii)
201         format(/1X,A,' Inconsistent final level:',
     $           /'     Gamma: ',F8.2,'. Initial level: 'A,1X,F8.2,
     $            ', final level: ',A,1X,F8.2)
            write(4,103) el(prev),del(prev),el(next),del(next)
103         format('     Previous/next final level:'2F10.3,' and',
     $            2F10.3)
         endif
      enddo

      return
      END

      SUBROUTINE EtoE2(INP,IOUT)
c  Reads ENSDF and writes ENSDF/2

      implicit none
      DOUBLE PRECISION etemp,dle,detemp,fmax,deg,demin
      logical SkipLevel
      integer i,ng,inp,iout,nn,ij,jj,numb,j
      integer nerr,max,nscore,nmat,nk,k,ik,itypes
c     Character strings for subroutine FEND:
      CHARACTER*2 NE*3
c     Local character strings:
      character*5 MLL,MGG,GMATCH*10,unit*3
c     Local card character strings
      character*80 ICARD
      character lflag(200)*1
      integer nlflag

      INCLUDE 'fconv.inc'

      nlflag=0
      NLEV=0
      NG=0
      NGAM=0
      NALPHA=0
      NBETA=0
      NPAR=0
      NBAND=0
      CPAR(1)='     '
      CPAR(2)='     '
      NewDS=.true.
      Sep=.false.

c  Read dataset
c  ************
98    NN=0
c     Loop through all cards of a data set
      do 500 i=1,50000
         NN=NN+1
         if(NN.GT.MCARD) call stopit(card(1),
     $    ' ** Too many cards **',' Increase MCARD',' ',NN)
         read(INP,'(A)',end=99) card(NN)

c        * For debugging *
c         write(*,'(1X,I4,'' '',A)') NN,card(NN)(1:70)

c        Skip blank cards
         if(i.eq.1.and.card(1).eq.' ') goto 98
         GOTO 101

c        End-of-file found
99       TheEnd=.TRUE.
         NN=NN-1
c        Take care of ending blank card
         if(NN.LE.1) NewDS=.false.
         go to 5000

101      continue
c  Find out what type of card it is
c  Data set identification
c  -----------------------
         IF(NN.eq.1) THEN
            if(card(NN)(6:9).NE.'   ') call stopit
     $         (card(1),' ** Invalid ID card **',' ',' ',0)

		  if(DSStrL.gt.0) then
			 if(index(card(1),DSStr(1:DSStrL)).eq.0) then
                  call skip(INP)
                  if(TheEnd) goto 99
                  goto 98
               endif
		  endif

            write(4,'(/,a)') card(NN)(1:80)
            card(NN)(9:9)='*'
            DSID=card(NN)(10:80)

c           Save NUCID. Get A and Z. Determine IODD.
            CALL ZNAME (card(NN)(4:5),Z,1)
            NUCID=card(NN)(1:5)
            READ (NUCID(1:3),'(I3)') A
            CALL ODDEVN (A,IODD)

c  Level card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  L ') THEN
            NLEV=NLEV+1
            if(NLEV.GT.MLEV) call stopit(card(1),
     $       ' ** Too many levels **',' Increase MLEV',' ',NLEV)
c           Save band head spin if first level in band
            do idum=1,NBand
               if(card(NN)(77:77).eq.BD(idum)) then
                  if(BdStr(idum).eq.' ') then
                     if(card(NN)(22:39).ne.' ') then
                        BdStr(idum)=card(NN)(22:39)
                     else
                        BdStr(idum)='?'
                     endif
                  endif
               endif
            enddo
c           Get energy
            CALL FEND(card(NN)(10:19),card(NN)(20:21),x,NE,els(NLEV))
            EL(NLEV)=x(1)
            DEL(NLEV)=x(2)
            MEL(NLEV)=' '
            Ban(NLEV)=' '

c           Calculate level energy if given as SP+x or SN+x
            call SpSn(card(NN)(10:19),SkipLevel)
            if(SkipLevel.and.IOUT.eq.0) then
               NLEV=NLEV-1
               goto 515
            endif

c           Save spin/parity and half-life, reset NDEC
            call GetJPi(card(NN)(22:39),spin(NLEV),parity(NLEV),
     $       jpimod(NLEV),JOffs(NLev))
            CALL FENDU(card(NN)(40:49),card(NN)(50:55),x,
     $       NE,ts(NLEV),unit)
            call un2sec(x,unit,ts(NLEV))
            t(NLEV)=x(1)
            dt(NLEV)=x(2)
            dt2(NLEV)=x(3)
            NDec(NLev)=0

c           Check level order
            IF(NLEV.GT.1.AND.EL(NLEV).LT.EL(NLEV-1).AND.EL(NLEV)
     $       .GE.1.0E-10) THEN
c              Allow E+X levels to be "out of order"
               IF(INDEX(card(NN)(10:19),'+').LE.0.AND.
     $          INDEX(card(L(NLEV-1))(10:19),'+').LE.0) THEN
                  call stopit(card(1),card(NN),card(L(NLEV-1)),
     $                        ' ** Level out of order **',0)
               ENDIF
            ENDIF
c           Save card number for later use
            L(NLEV)=NN

c           Left justify for FL= matching
            DO IJ=10,19
               IF(card(L(NLEV))(10:10).NE.' ') GO TO 513
               card(L(NLEV))(10:18)=card(L(NLEV))(11:19)
               card(L(NLEV))(19:19)=' '
            ENDDO
513         CONTINUE

c           +X goes to MEL array
            JJ=INDEX(card(NN)(10:19),'+')+9
c           Check for exponent, SP or SN
            if(JJ.GT.9.AND.card(NN)(JJ-1:JJ-1).NE.'E') then
               if(index(card(NN),'SP').le.0.and.
     $          index(card(NN),'SN').le.0) then
c                 Check for X+
                  IF(NUMB(card(NN)(JJ-1:JJ-1)).EQ.0.AND.card(NN)
     $             (JJ-1:JJ-1).NE.' ') MEL(NLEV)=card(NN)(JJ-1:JJ-1)
c                 Check for +X
                  IF(NUMB(card(NN)(JJ+1:JJ+1)).EQ.0.AND.card(NN)
     $             (JJ+1:JJ+1).NE.' ') MEL(NLEV)=card(NN)(JJ+1:JJ+1)
               endif
c              Check for letter by itself
            elseif(ITYPES(card(NN)(10:19)).ne.-2) then
               dumstr=card(NN)(10:19)
               call lbsup(dumstr)
               mel(NLEV)=dumstr(1:1)
            endif

c           Put col 80 (?) to col 1, col 78 (M) to col 2 and clear structure
c           flag (col. 3)
            UncLev(NLev)=card(NN)(80:80)
            card(NN)(1:1)=card(NN)(80:80)
            card(NN)(2:2)=card(NN)(78:78)
            card(NN)(78:80)='   '
            card(NN)(3:3)=' '
c           Add level ID to cols 4-5
            CALL CHARAD(NLEV,card(NN)(4:5))

c           Set stucture flag in col. 3 and other flags on FLAG= cont. card
            IF(card(NN)(77:77).NE.' ') THEN
c              Check if band with col. 77 tag is defined
               do 514 ij=1,NBAND
                  IF(BD(ij).NE.card(NN)(77:77)) GOTO 514
                  card(NN)(3:3)=BD(ij)
                  card(NN)(77:77)=' '
                  Ban(NLEV)=BD(IJ)
c                 If there is a level comment with the same tag, write
c                  a FLAG= card
                  do jj=1,nlflag
                     if (lflag(jj).eq.Ban(NLev)) then
                        NN=NN+1
                        card(NN)=NUCID//'F L FLAG='//card(NN-1)(3:3)
                        goto 515
                     endif
                  enddo
                  go to 515
514            continue
c              It is not a band; put it on FLAG= cont.card
               NN=NN+1
               card(NN)=NUCID//'F L FLAG='//card(NN-1)(77:77)
               card(NN-1)(77:77)=' '
            ENDIF
515         continue

c  Alpha card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  A ') THEN
            NALPHA=NALPHA+1
            card(NN)(1:5)='     '

c  EC/Beta+ card
c  -------------
         ELSEIF(card(NN)(6:9).EQ.'  E ') THEN
            NBETA=NBETA+1
            card(NN)(1:5)='     '

c  Beta- card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  B ') THEN
            NBETA=NBETA+1
            card(NN)(1:5)='     '

c  Parent card
c  -----------
         ELSEIF(card(NN)(6:9).EQ.'  P ') THEN
            NPAR=NPAR+1
c           Put parent Nucid in CPAR
            IF(NPar.LE.2) CPar(NPar)=card(NN)(1:5)

c  Q card
c  ------
         ELSEIF(card(NN)(6:9).EQ.'  Q ') THEN
c           Put Nucid in SNucId
            CALL ZNAME (card(NN)(4:5),idum,1)
            SNucId=card(NN)(1:5)
c           Save SP and SN with uncertainties
            CALL FEND(card(NN)(22:29),card(NN)(30:31),x,NE,dumstr)
            Sn=x(1)
            DSn=x(2)
            CALL FEND(card(NN)(32:39),card(NN)(40:41),x,NE,dumstr)
            Sp=x(1)
            DSp=x(2)

c  End card
c  --------
         ELSEIF(card(NN).EQ.' ') THEN
            GO TO 5000

c  Gamma card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  G ') THEN
c           Clear NUCID field. Move ? flag to col. 1. Clear coin field.
            card(NN)(1:5)='     '
            NERR=0
            card(NN)(1:1)=card(NN)(80:80)
            card(NN)(78:80)='   '

c           Ignore unplaced gammas and decay from levels with E=0
            IF(NLEV.EQ.0) GO TO 508
            if(el(NLEV).le.0) then
               write(4,211) card(NN)(10:19),card(L(NLEV))(10:19)
211            format(/'*E* Gamma depopulating level with E=0'
     $                /' Gamma ',A,' from level ',A,' ignored')
               go to 508
            endif

c           Band assignment code from initial level to gamma record col. 2
            card(NN)(2:2)=card(L(NLEV))(3:3)
            NG=NN
            NGAM=NGAM+1
            if(NGAM.GT.MGAM) call stopit(card(1),
     $       ' ** Too many gammas **',' Increase MGAM',' ',NGAM)
            inlev(ngam)=NLEV

            UncTr(NGam)=card(NN)(1:1)
c           Get energy, default DE for matching is 1 keV
            CALL fend(card(NN)(10:19),card(NN)(20:21),x,NE,egams(NGAM))
            egam(NGAM)=x(1)
            degam(NGAM)=x(2)
            DEG=degam(NGAM)
            IF(degam(NGAM).LT.1.0D-10) DEG=1.0
            ETEMP=EL(NLEV)-egam(NGAM)
            DLE=DEL(NLEV)
            IF(DLE.LT.1.0D-10) DLE=1.0
            DETEMP=DSQRT(DEG**2+DLE**2)
c           Count the gammas from this level
            NDEC(NLEV)=NDEC(NLEV)+1

c           Get intensity
            CALL fend(card(NN)(22:29),card(NN)(30:31),x,NE,igams(NGAM))
            igam(NGAM)=x(1)
            digam(NGAM)=x(2)

c           Get multipolarity
            MULT(NGAM)=card(NN)(32:41)
            call GetML(mult(NGam),ML(1,NGam),MLPar(NGam),MLMod(NGam))

c           Get mixing ratio
            ds(NGam)='  '
            CALL fend(card(NN)(42:49),card(NN)(50:55),x,NE,ds(NGAM))
            d(NGAM)=x(1)
            dd(NGAM)=x(2)
            dd2(NGAM)=x(3)
c           If + sign is given, save it in DSign
            DSign(NGam)=' '
            dumstr=card(NN)(42:49)
            call lbsup(dumstr)
            if(dumstr(1:1).eq.'+')  DSign(NGam)='+'

            MAX=1
            NSCORE=0
            DEMIN=1000.0
            NMAT=0

c           Score match on basis of energy, Mult-JPi, band assignment, and +XYZ
            DO 541 IJ=1,NLEV-1
               ISCORE(IJ)=0
c              Test only cases where +XYZ are identical or absent
               IF(MEL(IJ).NE.MEL(NLEV)) GO TO 541
               CALL ESCORE(ETEMP,DETEMP,EL(IJ),DEL(IJ),ISCORE(IJ))

c              If no energy match give up and try again
               IF(ISCORE(IJ).EQ.0) GO TO 541

c              Check for common band assignment, add 2 to score
               IF(card(L(NLEV))(3:3).EQ.card(L(IJ))(3:3).AND.
     $          card(L(NLEV))(3:3).NE.' ') ISCORE(IJ)=ISCORE(IJ)+2

c              Check for consistent multipolarity and JPI:s
               CALL MJSCOR(card(NN)(32:41),card(L(NLEV))(22:39),
     $          card(L(IJ))(22:39),IODD,ISCORE(IJ))
               IF(ISCORE(IJ).LE.0) GO TO 541
               NMAT=NMAT+1

c              MAX is level with best fit (highest score)
               IF(ISCORE(IJ).LT.NSCORE) GO TO 541
c              Two levels with same score, choose closest
               IF(ISCORE(IJ).EQ.NSCORE) THEN
                  IF(abs(ETEMP-EL(IJ)).lt.DEMIN) THEN
                     MAX=IJ
                     DEMIN=abs(ETEMP-EL(MAX))
                     NSCORE=ISCORE(IJ)
                  ENDIF
c              Two levels with different score, choose best score
               ELSE
                  MAX=IJ
                  DEMIN=abs(ETEMP-EL(MAX))
                  NSCORE=ISCORE(IJ)
               ENDIF
541         CONTINUE

            if(nscore.gt.0) then
               card(NN)(4:5)=card(l(max))(4:5)
               card(NN)(3:3)=Ban(max)
               finlev(ngam)=MAX
               GO TO 542
            endif

c           If no fits get a score, write to output file for further review and
c           choose closest level
            CALL MATCH(ETEMP,EL,NLEV,MAX)
c           Make sure final level is not initial level
            if(MAX.EQ.NLEV) MAX=MAX0(NLEV-1,1)
            card(NN)(4:5)=card(L(MAX))(4:5)
            card(NN)(3:3)=Ban(max)
            finlev(ngam)=MAX

C           If closest level is only reasonable fit, all is OK
            FMAX=5.0*DETEMP
            IF(FMAX.LT.5.0) FMAX=5.0
            IF(MAX.EQ.1.AND.EL(MAX+1)-EL(MAX).GT.FMAX) GO TO 542
            IF(MAX.GT.1.AND.EL(MAX+1)-EL(MAX).GT.FMAX.AND.EL(MAX)-
     $       EL(MAX-1).GT.FMAX) GO TO 542
            if(logon.ge.1) then
               WRITE(4,255) card(NN)(10:49),card(L(NLEV))(3:3),
     $          card(L(NLEV))(10:49),ETEMP
               WRITE(4,256) card(L(MAX))(3:3),card(L(MAX))(10:56)
256            FORMAT(6X,'# Matched to level  ',A,' ',A)
            endif
            NERR=1

            if(logon.ge.1) then
               IF(MAX.EQ.1) WRITE(4,257) card(L(2))(3:3),
     $          card(L(2))(10:56)
               IF(MAX.GT.1) WRITE(4,257)
     $          card(L(MAX-1))(3:3),card(L(MAX-1))(10:56),
     $          card(L(MAX+1))(3:3),card(L(MAX+1))(10:56)
257            FORMAT(7X,' Nearby levels are ',A,' ',A,
     $               /7X,' and               ',A,' ',A)
            endif
            GO TO 511

542         CONTINUE

            card(NN)(4:5)=card(L(MAX))(4:5)
            card(NN)(3:3)=Ban(max)
            finlev(ngam)=MAX

C           If multiple possibilities write all to output
            IF(NMAT.LE.1) GO TO 511
            if(logon.ge.1) then
               WRITE(4,255) card(NN)(10:49),card(L(NLEV))(3:3),
     $          card(L(NLEV))(10:49),ETEMP
255            FORMAT(/' Gamma with E=',A
     $                /'  from ',A,' level with E=',A,' DeltaE=',F8.2)
            endif
            DO IJ=1,NLEV-1
               IF(ISCORE(IJ).GT.0.and.logon.ge.1) THEN
                  WRITE(4,407) card(L(IJ))(3:3),
     $                         card(L(IJ))(10:49),ISCORE(IJ)
407               FORMAT(14X,A,' level=',A,7X,'Score=',I3)
                  NERR=1
               ENDIF
            ENDDO
            if(logon.ge.1) WRITE(4,'(''      # Level chosen: '',A)')
     $       card(L(MAX))(10:49)

511         CONTINUE

508         CONTINUE

c           Put col. 77 flag on FLAG= cont.card
            if(card(NN)(77:77).ne.' ') then
               NN=NN+1
               card(NN)='     '//'F G FLAG='//card(NN-1)(77:77)
               card(NN-1)(77:77)=' '
            endif

c  Final level flag (FL=) on gamma continuation card
c  -------------------------------------------
      ELSEIF(card(NN)(7:9).EQ.' G '.AND.INDEX(card(NN),'FL=').GT.0) THEN
         JJ=INDEX(card(NN),'FL=')+2
         IF(JJ.LE.2) GO TO 512
         GMATCH='          '
         NK=0
         DO K=1,12
c           Skip blanks. Put FL value left justified in GMATCH
            IF(card(NN)(JJ+K:JJ+K).NE.' ') THEN
               IF(card(NN)(JJ+K:JJ+K).EQ.'$') GO TO 523
               NK=NK+1
               GMATCH(NK:NK)=card(NN)(JJ+K:JJ+K)
            ENDIF
         ENDDO
523      CONTINUE

c        Compare previous levels with GMATCH
         DO IJ=1,NLEV-1
            IF(card(L(IJ))(10:19).EQ.GMATCH) THEN
c              Match found
               card(NG)(4:5)=card(L(IJ))(4:5)
               card(NN)(3:3)=Ban(IJ)
               finlev(ngam)=IJ
c              Do not output FL= second card
               NN=NN-1
               IF(NERR.EQ.1) THEN
                  if(logon.ge.1) WRITE(4,259) GMATCH
259               FORMAT('  & Problem resolved by FL=',A)
               ENDIF
               GO TO 512
            ENDIF
         ENDDO
c        Error message for no match found
         WRITE(4,1525) '*E*',GMATCH,card(NG)(1:22)
1525     FORMAT(/1X,A,' No FL=',A,' match for ',A)
c        Do not output FL= second card
         NN=NN-1

512      CONTINUE

c  Band (level) comment
c  --------------------
      ELSEIF((card(NN)(6:9).EQ.' CL '.OR.card(NN)(6:9).eq.' cL ')
     $ .AND.INDEX(card(NN)(10:20),'BAND(').GT.0) THEN
         JJ=INDEX(card(NN),'BAND(')+4
         IF(JJ.LE.4) GO TO 510

         do j=1,10
            IF(card(NN)(JJ+j:JJ+j).EQ.')'.or.
     $       card(NN)(JJ+j:JJ+j).EQ.' ') GOTO 510

            if(card(NN)(JJ+j:JJ+j).ne.',') then
               NBAND=NBAND+1
               IF(NBAND.GT.MBAND) call stopit(card(1),
     $          ' ** Too many bands **',' Increase MBAND',' ',NBAND)
               BD(NBAND)=card(NN)(JJ+j:JJ+j)
c              Clear band head JPi field
               BdStr(NBand)=' '

c              Clear col:s 1-5 and put band flag in col. 3
               card(NN)(1:5)='     '
               card(NN)(3:3)=BD(NBAND)

c              Save comment text as band identification
               IJ=INDEX(card(NN),'$')+1
               IF(IJ.le.1.or.IJ.gt.20) IJ=20
               BANDID(NBAND)=card(NN)(IJ:80)
            endif
         enddo

510      CONTINUE

c  Band flag on level continuation card (FLAG=)
c  --------------------------------------------
         ELSEIF(card(NN)(7:9).EQ.' L '.AND.INDEX(card(NN),'FLAG=').GT.0)
     $    THEN
            IF(card(L(NLEV))(3:3).NE.' ') go to 509
            JJ=INDEX(card(NN),'FLAG=')+3
            IF(JJ.LE.3) GO TO 509
            DO IJ=2,10
               IF(card(NN)(JJ+IJ:JJ+IJ).EQ.'$') GO TO 509
               IF(card(NN)(JJ+IJ:JJ+IJ).NE.' '.AND.
     $          card(NN)(JJ+IJ:JJ+IJ).NE.',') THEN
                  DO IK=1,NBAND
                     IF(BD(IK).EQ.card(NN)(JJ+IJ:JJ+IJ)) THEN
                        card(L(NLEV))(3:3)=BD(IK)
                        Ban(NLEV)=BD(IK)
c                       Save band head spin if first level in band
                        if(BdStr(IK).eq.' ') then
                           if(card(L(NLev))(22:39).ne.' ') then
                              BdStr(IK)=card(L(NLev))(22:39)
                           else
                              BdStr(IK)='?'
                           endif
                        endif
                        GO TO 509
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO

509         CONTINUE

c  Level comment which is NOT a band comment
c  -----------------------------------------
         ELSEIF((card(NN)(6:9).EQ.' CL '.OR.card(NN)(6:9).eq.' cL ')
     $    .AND.INDEX(card(NN)(10:20),'(').GT.0) THEN
            JJ=INDEX(card(NN),'(')+1
            nlflag=nlflag+1
            lflag(nlflag)=card(NN)(JJ:JJ)
            card(NN)(1:5)='     '

c  Remove nucid on any other card
c  ------------------------------
         ELSE
            card(NN)(1:5)='     '
         ENDIF

500   CONTINUE

c  Prepare and write data to ENSDF/2 file
c  **************************************
5000  CONTINUE
      IF(.NOT.NewDS) RETURN

c     Do not write ENSDF/2 data set if IOUT is 0
      IF(IOUT.EQ.0) RETURN

c     Write ID card
      WRITE(IOUT,'(A)') card(1)

c     Prepare information (I) card
      ICARD=' '
      IF(card(1)(4:5).NE.'  ') THEN
         Icard(1:3)=card(1)(1:3)
         WRITE(Icard(4:6),'(I3)') Z
         Icard(8:8)='I'
         WRITE(Icard(11:14),'(I4)') NLEV
         WRITE(Icard(21:24),'(I4)') NGAM
         WRITE(Icard(31:34),'(I4)') NALPHA
         WRITE(Icard(41:44),'(I4)') NBETA
         WRITE(Icard(51:54),'(I4)') NPAR
         Icard(55:59)=CPAR(1)
         Icard(60:64)=CPAR(2)
         WRITE(Icard(71:74),'(I4)') NBAND
         WRITE(IOUT,'(A)') Icard(1:74)
      ENDIF

      MLL='     '
      MGG='     '
      DO IJ=2,NN
         IF(card(IJ)(6:9).EQ.'  L ') MLL=card(IJ)(1:5)
         IF(card(IJ)(6:9).EQ.'  G ') MGG=card(IJ)(1:5)

c  Find last non-blank character
         DO IK=1,80
            IF(card(IJ)(81-IK:81-IK).NE.' ') GO TO 5003
         ENDDO
5003     CONTINUE
         IK=81-IK

c  Make sure level/gamma comments and cont. cards have level and band info
         IF(card(IJ)(8:8).EQ.'L'.and.MLL.ne.' ') card(IJ)(1:5)=MLL
         IF(card(IJ)(8:8).EQ.'G'.and.MGG.ne.' ') card(IJ)(1:5)=MGG

c  Write cards to output
         if(IK.gt.0) then
            WRITE(IOUT,'(A)') card(IJ)(1:IK)
         else
            WRITE(IOUT,'(A)') ' '
         endif

      ENDDO
      return
      END

      SUBROUTINE E2toE(INP,IOUT)
c  Reads ENSDF/2 and writes ENSDF

      implicit none
      logical FLlog
      logical SkipLevel
      integer i,ng,inp,iout,nn,ij,jj,numb,j
      integer k,ik,ITYPES
c     Character strings for subroutine FEND:
      CHARACTER*2 NE*3,unit*3

      INCLUDE 'fconv.inc'

      NLEV=0
      NG=0
      NGAM=0
      NALPHA=0
      NBETA=0
      NPAR=0
      NBAND=0
      CPAR(1)='     '
      CPAR(2)='     '
      NewDS=.true.
      Sep=.false.

c  Read dataset
c  ************
98    NN=0
c     Loop through all cards of a data set
      do 500 i=1,50000
         NN=NN+1
         if(NN.GT.MCARD) call stopit(card(1),
     $    ' ** Too many cards **',' Increase MCARD',' ',NN)
         read(INP,'(A)',end=99) card(NN)

c        * For debugging *
c         write(*,'(1X,I4,'' '',A)') NN,card(NN)(1:70)

c        Skip blank cards
         if(i.eq.1.and.card(1).eq.' ') goto 98
         GOTO 101

c        End-of-file found
99       TheEnd=.TRUE.
         NN=NN-1
c        Take care of ending blank card
         if(NN.LE.1) NewDS=.false.
         go to 5000

101      continue
c  Find out what type of card it is
c  Data set identification
c  -----------------------
         IF(NN.eq.1) THEN
            if(card(1)(6:9).NE.'   *') call stopit
     $         (card(1),' ** Invalid ID card **',' ',' ',0)

            if(DSStrL.gt.0) then       
		     if(index(card(1),DSStr(1:DSStrL)).eq.0) then
                  call skip(INP)
                  if(TheEnd) goto 99
                  goto 98
               endif
		  endif

            card(NN)(9:9)=' '
            write(4,'(/,a)') card(NN)(1:80)
            DSID=card(NN)(10:80)

c           Save NUCID. Get A and Z. Determine IODD.
            CALL ZNAME (card(NN)(4:5),Z,0)
c           Translate to old ENSDF convention with upper case and 04 for Rf
            CALL ZNAME (card(NN)(4:5),Z,3)
            NUCID=card(NN)(1:5)
            read (NUCID(1:3),'(I3)') A
            CALL ODDEVN (A,IODD)

c  Data set information card
c  -------------------------
         ELSEIF(card(NN)(7:9).eq.' I ') THEN
c           Skip this card for ENSDF format
            NN=NN-1

c  Level card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  L ') THEN
            NLEV=NLEV+1
            if(NLEV.GT.MLEV) call stopit(card(1),
     $       ' ** Too many levels **',' Increase MLEV',' ',NLEV)
c           Save band head spin if first level in band
            do idum=1,NBand
               if(card(NN)(3:3).eq.BD(idum)) then
                  if(BdStr(idum).eq.' ') then
                     if(card(NN)(22:39).ne.' ') then
                        BdStr(idum)=card(NN)(22:39)
                     else
                        BdStr(idum)='?'
                     endif
                  endif
               endif
            enddo

c           Get energy
            CALL FEND(card(NN)(10:19),card(NN)(20:21),x,NE,els(NLEV))
            EL(NLEV)=x(1)
            DEL(NLEV)=x(2)
            MEL(NLEV)=' '
            Ban(NLEV)=' '

c           Calculate level energy if given as SP+x or SN+x
            call SpSn(card(NN)(10:19),SkipLevel)
            if(SkipLevel.and.IOUT.eq.0) then
               NLEV=NLEV-1
               goto 515
            endif


c           Save spin/parity and half-life, reset NDEC
            call GetJPi(card(NN)(22:39),spin(NLEV),parity(NLEV),
     $       jpimod(NLEV),JOffs(NLev))

            CALL FENDU(card(NN)(40:49),card(NN)(50:55),x,
     $       NE,ts(NLEV),unit)
            call un2sec(x,unit,ts(NLEV))
            t(NLEV)=x(1)
            dt(NLEV)=x(2)
            dt2(NLEV)=x(3)
            NDEC(NLEV)=0

c           Check level order (only if IOUT>0)
            IF(NLEV.GT.1.AND.EL(NLEV).LT.EL(NLEV-1).AND.EL(NLEV)
     $       .GE.1.0E-10) THEN
c              Allow E+X levels to be "out of order"
               IF(INDEX(card(NN)(10:19),'+').LE.0.AND.
     $          INDEX(card(L(NLEV-1))(10:19),'+').LE.0) THEN
                  IF(IOUT.gt.0) then
                     call stopit(card(1),card(NN),card(L(NLEV-1)),
     $                           ' ** Level out of order **',0)
                  ELSE
                     write(4,*) 
                     write(4,*) '*E* Level out of order:'
                     write(4,*) ' ',card(NN)
                     write(4,*)
                  ENDIF
               ENDIF
            ENDIF
c           Save pointer for level card of level
            L(NLEV)=NN

c           Left justify
            DO IJ=10,19
               IF(card(L(NLEV))(10:10).NE.' ') GO TO 513
               card(L(NLEV))(10:18)=card(L(NLEV))(11:19)
               card(L(NLEV))(19:19)=' '
            ENDDO
513         CONTINUE

c           +X goes to MEL array
            JJ=INDEX(card(NN)(10:19),'+')+9
c           Check for exponent, SP or SN
            if(JJ.GT.9.AND.card(NN)(JJ-1:JJ-1).NE.'E') then
               if(index(card(NN),'SP').le.0.and.
     $          index(card(NN),'SN').le.0) then
c                 Check for X+
                  IF(NUMB(card(NN)(JJ-1:JJ-1)).EQ.0.AND.card(NN)
     $             (JJ-1:JJ-1).NE.' ') MEL(NLEV)=card(NN)(JJ-1:JJ-1)
c                 Check for +X
                  IF(NUMB(card(NN)(JJ+1:JJ+1)).EQ.0.AND.card(NN)
     $             (JJ+1:JJ+1).NE.' ') MEL(NLEV)=card(NN)(JJ+1:JJ+1)
               endif
c              Check for letter by itself
            elseif(ITYPES(card(NN)(10:19)).ne.-2) then
               dumstr=card(NN)(10:19)
               call lbsup(dumstr)
               mel(NLEV)=dumstr(1:1)
            endif

c           Col. 1 to col 80, col 2 to col 78
c               (?)               isomer flag (M)
            card(NN)(80:80)=card(NN)(1:1)
            UncLev(NLev)=card(NN)(80:80)
            card(NN)(78:78)=card(NN)(2:2)

            IJ=NN
c           Set stucture flag (col. 3) in col. 77 if available
            IF(card(NN)(77:77).EQ.' '.AND.card(NN)(3:3).NE.' ') THEN
               card(NN)(77:77)=card(NN)(3:3)
               Ban(NLEV)=card(NN)(3:3)
c           Write level comment flag when pos 77 is taken
            ELSEIF(card(NN)(77:77).NE.' '.AND.card(NN)(3:3).NE.' ') THEN
               if(IOUT.gt.0) then
                  NN=NN+1
                  card(NN)=NUCID//'2 L FLAG='//card(NN-1)(3:3)
                  Ban(NLEV)=card(NN-1)(3:3)
               endif
            ENDIF
            card(IJ)(1:5)=NUCID

515      continue
c  Alpha card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  A ') THEN
            NALPHA=NALPHA+1
            card(NN)(1:5)=NUCID

c  EC/Beta+ card
c  -------------
         ELSEIF(card(NN)(6:9).EQ.'  E ') THEN
            NBETA=NBETA+1
            card(NN)(1:5)=NUCID

c  Beta- card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  B ') THEN
            NBETA=NBETA+1
            card(NN)(1:5)=NUCID

c  Parent card
c  -----------
         ELSEIF(card(NN)(6:9).EQ.'  P ') THEN
            NPAR=NPAR+1
c           Put parent Nucid in CPAR
            IF(NPAR.LE.2) CPAR(NPAR)=card(NN)(1:5)

c  Q card
c  ------
         ELSEIF(card(NN)(6:9).EQ.'  Q ') THEN
c           Put Nucid in SNucId
            CALL ZNAME (card(NN)(4:5),idum,0)
            CALL ZNAME (card(NN)(4:5),idum,3)
            SNucId=card(NN)(1:5)
c           Save SP and SN with uncertainties
            CALL FEND(card(NN)(22:29),card(NN)(30:31),x,NE,dumstr)
            SN=x(1)
            DSN=x(2)
            CALL FEND(card(NN)(32:39),card(NN)(40:41),x,NE,dumstr)
            SP=x(1)
            DSP=x(2)

c  End card
c  --------
         ELSEIF(card(NN).EQ.' ') THEN
            GO TO 5000

c  Gamma card
c  ----------
         ELSEIF(card(NN)(6:9).EQ.'  G ') THEN
            IJ=NN
c           Restore ? flag to col. 80
            card(NN)(80:80)=card(NN)(1:1)

c           Ignore unplaced gammas and decay from levels with E=0
            IF(NLEV.EQ.0) GO TO 511
            if(el(NLEV).le.0) then
               write(4,211) card(NN)(10:19),card(L(NLEV))(10:19)
211            format(/' Gamma ',A,' from level ',A,' ignored')
               go to 511
            endif

c           Increment gamma counter, save initial level
            NGAM=NGAM+1
            if(NGAM.GT.MGAM) call stopit(card(1),
     $       ' ** Too many gammas **',' Increase MGAM',' ',NGAM)
            inlev(ngam)=NLEV
            UncTr(NGam)=card(NN)(80:80)

c           Get energy
            CALL fend(card(NN)(10:19),card(NN)(20:21),x,NE,egams(NGAM))
            egam(NGAM)=x(1)
            degam(NGAM)=x(2)
c           Count the gammas from this level
            NDEC(NLEV)=NDEC(NLEV)+1

c           Get intensity
            CALL fend(card(NN)(22:29),card(NN)(30:31),x,NE,igams(NGAM))
            igam(NGAM)=x(1)
            digam(NGAM)=x(2)

c           Get multipolarity
            MULT(NGAM)=card(NN)(32:41)
            call GetML(mult(NGam),ML(1,NGam),MLPar(NGam),MLMod(NGam))

c           Get mixing ratio
            ds(NGam)='  '
            CALL fend(card(NN)(42:49),card(NN)(50:55),x,NE,ds(NGAM))
            d(NGAM)=x(1)
            dd(NGAM)=x(2)
            dd2(NGAM)=x(3)
c           Save + sign in DSign
            DSign(NGam)=' '
            dumstr=card(NN)(42:49)
            call lbsup(dumstr)
            if(dumstr(1:1).eq.'+')  DSign(NGam)='+'

c           Find out final level number (K), save final level
            call icharad(card(NN)(4:5),K)
            finlev(ngam)=K

c           Add FL= comment as next card if neighbouring levels within 5 keV
            FLlog=.false.
            if(K.ge.2.and.abs((el(K)-el(K-1))).lt.5) FLlog=.true.
            if(K.le.(NLEV-1).and.abs((el(K)-el(K+1))).lt.5) FLlog=.true.
            IF(FLlog) THEN
               if(IOUT.gt.0) then
                  NN=NN+1
                  card(NN)=NUCID//'2 G '//'FL='//card(L(K))(10:19)
               endif
            ENDIF

511         CONTINUE
c           Restore NUCID
            card(IJ)(1:5)=NUCID

c  Band (level) comment
c  --------------------
         ELSEIF((card(NN)(6:9).EQ.' CL '.OR.card(NN)(6:9).eq.' cL ')
     $    .AND.index(card(NN)(10:20),'BAND(').GT.0) THEN
            JJ=INDEX(card(NN),'BAND(')+4
            IF(JJ.LE.4) GO TO 512

            do j=1,10
               IF(card(NN)(JJ+j:JJ+j).EQ.')'.or.
     $          card(NN)(JJ+j:JJ+j).EQ.' ') GOTO 512

               if(card(NN)(JJ+j:JJ+j).ne.',') then
                  NBAND=NBAND+1
                  IF(NBAND.GT.MBAND) call stopit(card(1),
     $             ' ** Too many bands **',' Increase MBAND',' ',NBAND)
                  BD(NBAND)=card(NN)(JJ+j:JJ+j)
c                 Clear band head JPi field
                  BdStr(NBand)=' '

c                 Save comment text as band identification
                  IJ=INDEX(card(NN),'$')+1
                  IF(IJ.le.1.or.IJ.gt.20) IJ=20
                  BANDID(NBAND)=card(NN)(IJ:80)
               endif
            enddo
512         CONTINUE
            card(NN)(1:5)=NUCID

c  Add nucid to any other card
c  ------------------------------
         ELSE
            card(NN)(1:5)=NUCID
         ENDIF

500   CONTINUE

c  Prepare and write data to ENSDF file
c  ************************************
5000  CONTINUE
      IF(.NOT.NewDS) RETURN

c     Do not write ENSDF data set if IOUT is 0
      IF(IOUT.EQ.0) RETURN

c     Write ID card
      WRITE(IOUT,'(A)') card(1)

      DO IJ=2,NN
c  Find last non-blank character
         DO 5002 IK=1,80
            IF(card(IJ)(81-IK:81-IK).NE.' ') GO TO 5003
5002     CONTINUE
5003     CONTINUE
         IK=81-IK

c  Write cards to output
         if(IK.gt.0) then
            WRITE(IOUT,'(A)') card(IJ)(1:IK)
         else
            WRITE(IOUT,'(A)') ' '
         endif

      ENDDO
      return
      END

      SUBROUTINE WENSDF2(OUT)
c  Writes ENSDF/2 data using common block data

      implicit none
      integer OUT,i,ii,j
      character*80 ocard,unit*3

      INCLUDE 'fconv.inc'

c     Write ID card
      WRITE(ocard(1:80),'(A5,''   *'',A71)') nucid(1:5),dsid(1:71)
      call cardout(out,ocard)

c     Prepare information (I) card. Write it.
      ocard=' '
      write(ocard(1:3),'(I3)') A
      write(ocard(4:6),'(I3)') z
      ocard(8:8)='I'
      write(ocard(11:14),'(I4)') NLev
      write(ocard(21:24),'(I4)') NGam
      write(ocard(31:34),'(I4)') NAlpha
      write(ocard(41:44),'(I4)') NBeta
      write(ocard(51:54),'(I4)') NPar
      ocard(55:59)=cpar(1)
      ocard(60:64)=cpar(2)
      write(ocard(71:74),'(I4)') NBand
      call cardout(out,ocard)

c     Write exchange comment
      if(LComm.gt.0) then
         ocard='      C  '//Comm(1:LComm)
         call cardout(out,ocard)
      endif

c  Write band comments
      do i=1,NBAND
         write(ocard(1:80),'(2X,A,3X,''CL BAND('',A'')$'',A)')
     $         BD(i),BD(i),BANDID(i)
         call cardout(out,ocard)
      enddo

c  Write levels/gammas
      do i=1,NLev
c        Prepare level card
         ii=loo(i)
         if(ii.le.0) goto 10
         ocard='       L '
         ocard(1:1)=UncLev(ii)
         ocard(3:3)=Ban(ii)
         call charad(lo(ii),ocard(4:5))

c        Level energy
         x(1)=el(ii)
         x(2)=del(ii)
         x(3)=0
         CALL WEND(x,ocard(10:19),10,ocard(20:21),2,els(ii),' ')
         call tbsup (ocard(10:19))
         if(mel(ii).ne.' ') call append(ocard(10:19),'+'//mel(ii))
         call padrt (ocard(10:19),10)

c        JPi
         call PutJPi(ocard(23:39),spin(ii),parity(ii),jpimod(ii),iodd,
     $               JOffs(ii))

c        T1/2
         if(t(ii).gt.0) then
            x(1)=t(ii)
            x(2)=dt(ii)
            x(3)=dt2(ii)
            call sec2un(x,unit)
            CALL WEND(x,ocard(40:49),10,ocard(50:55),6,ts(ii),unit)
         endif

c        Write level card
         call cardout(out,ocard)

         do j=1,ngam
            if(inlev(j).eq.ii) then
c              Prepare gamma card
               ocard='       G '
               ocard(1:1)=UncTr(j)
               ocard(2:2)=Ban(inlev(j))
               ocard(3:3)=Ban(finlev(j))
               call charad(lo(finlev(j)),ocard(4:5))
c              E(gamma)
               x(1)=egam(j)
               x(2)=degam(j)
               x(3)=0
               CALL WEND(X,ocard(10:19),10,ocard(20:21),2,egams(j),
     $          ' ')

c              Intensity of gamma if it is > 0.0
               if(igam(j).gt.0.0) then
                  x(1)=igam(j)
                  x(2)=digam(j)
                  x(3)=0
                  CALL WEND(x,ocard(22:29),8,ocard(30:31),2,igams(j),
     $             ' ')
               endif

c              Multipolarity
               call PutML(Mult(j),ML(1,j),MLPar(j),MLMod(j))
               ocard(32:41)=Mult(j)

c              Mixing ratio of gamma if it or error is .ne. 0.0
               if(d(j).ne.0.or.dd(j).ne.0) then
                  x(1)=d(j)
                  x(2)=dd(j)
                  x(3)=dd2(j)
                  CALL WEND(x,ocard(42:49),8,ocard(50:55),6,ds(j),
     $             ' ')
                  dumstr(1:7)=ocard(42:48)
                  if(DSign(j).eq.'+') ocard(42:49)='+'//dumstr
               endif

c              Write gamma card
               call cardout(out,ocard)
            endif

         enddo
10       continue
      enddo

c  Blank card as data set separator
      write(out,*) ' '

      return
      END

      SUBROUTINE WENSDF(OUT)
c  Writes ENSDF data using common block data

      implicit none
      integer OUT,i,ii,j,jj,jjj
      character*80 ocard,dum*2,unit*3
      logical FLlog

      INCLUDE 'fconv.inc'

c     Fix NUCID, Write ID card
      CALL ZNAME(NucID(4:5),Z,3)
      WRITE(ocard(1:80),'(A,4X,A)') nucid(1:5),dsid(1:71)
      call cardout(out,ocard)

c     Write exchange comment
      if(LComm.gt.0) then
         ocard=NUCID//' C  '//Comm(1:LComm)
         call cardout(out,ocard)
      endif

c  Write band comments
      do i=1,NBAND
         write(ocard(1:80),'(A,1X,''CL BAND('',A'')$'',A)')
     $         NUCID,BD(i),BANDID(i)
         call cardout(out,ocard)
      enddo

c  Write levels/gammas
      do i=1,nlev
c        Prepare level card
         ii=loo(i)
         if(ii.le.0) goto 10
         ocard=NUCID//'  L '

c        Level energy
         x(1)=el(ii)
         x(2)=del(ii)
         x(3)=0
         CALL WEND(x,ocard(10:19),10,ocard(20:21),2,els(ii),' ')
         call tbsup (ocard(10:19))
         if(mel(ii).ne.' ') call append(ocard(10:19),'+'//mel(ii))
         call padrt (ocard(10:19),10)

c        JPi
         call PutJPi(ocard(23:39),spin(ii),parity(ii),jpimod(ii),iodd,
     $               JOffs(ii))

c        T1/2 if it is known
         if(t(ii).gt.0) then
            x(1)=t(ii)
            x(2)=dt(ii)
            x(3)=dt2(ii)
            call sec2un(x,unit)
            CALL WEND(x,ocard(40:49),10,ocard(50:55),6,ts(ii),unit)
         endif

c        Set stucture flag (col. 3) in col. 77 and ? tag in col. 80
         ocard(77:77)=Ban(ii)
         ocard(80:80)=UncLev(ii)

c        Write level card
         call cardout(out,ocard)

         do j=1,ngam
            if(inlev(j).eq.ii) then
c              Prepare gamma card
               ocard=NUCID//'  G '

c              E(gamma)
               x(1)=egam(j)
               x(2)=degam(j)
               x(3)=0
               CALL WEND(x,ocard(10:19),10,ocard(20:21),2,egams(j),
     $          ' ')

c              Intensity of gamma if it is > 0.0
               if(igam(j).gt.0.0) then
                  x(1)=igam(j)
                  x(2)=digam(j)
                  x(3)=0
                  CALL WEND(x,ocard(22:29),8,ocard(30:31),2,igams(j),
     $             ' ')
               endif

c              Multipolarity
               call PutML(Mult(j),ML(1,j),MLPar(j),MLMod(j))
               ocard(32:41)=Mult(j)

c              Mixing ratio of gamma if it or error is .ne. 0.0
               if(d(j).ne.0.or.dd(j).ne.0) then
                  x(1)=d(j)
                  x(2)=dd(j)
                  x(3)=dd2(j)
                  CALL WEND(x,ocard(42:49),8,ocard(50:55),6,ds(j),
     $             ' ')
                  dumstr(1:7)=ocard(42:48)
                  if(DSign(j).eq.'+') ocard(42:49)='+'//dumstr
               endif
c              Write uncertainty tag in col. 80
               ocard(80:80)=UncTr(j)

c              Write gamma card
               call cardout(out,ocard)

c              Add FL= comment as next card if neighbouring final
c              levels are within 5 keV
               FLlog=.false.
               jj=finlev(j)
               if(lo(jj).ge.2) then
                  jjj=loo(lo(jj)-1)
                  if(abs((el(jj)-el(jjj))).lt.5) FLlog=.true.
               endif
               if(lo(jj).le.(NLEV-1)) then
                  jjj=loo(lo(jj)+1)
                  if(abs((el(jj)-el(jjj))).lt.5) FLlog=.true.
               endif
               IF(FLlog) THEN
                  ocard=NUCID//'2 G '//'FL='
                  x(1)=el(jj)
                  x(2)=del(jj)
                  x(3)=0
                  CALL WEND(x,ocard(13:22),10,dum,2,' ',' ')
                  call cardout(out,ocard)
               ENDIF

            endif

         enddo
10       continue
      enddo

c  Blank card as data set separator
      write(out,*) ' '

      return
      END

      SUBROUTINE ODDEVN(A,IODD)
C   DETERMINES IF ENSDF DATASET HAS ODD-A OR EVEN-A
      implicit none
      integer A,IODD,NA
C     A odd: NA=1 IODD=2
C     A even: NA=0 IODD=1

      IODD=2
      NA=(A+1)/2-A/2
      IF(NA.EQ.0) IODD=1
      RETURN
      END


      SUBROUTINE ESCORE(E,DE,X,DX,ISCORE)
c  Decide on final level for gamma
c  Score energy. Small uncertainty, small deviation adds 9 to score

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

c     Calculate uncertainty in deviation
      DZ=(DE**2+DX**2)**0.5
      IF(DZ.LT.0.0001) DZ=1.0
      MEZ=0
      JSCORE=0
c     Z is deviation in sigmas
      Z=DABS((E-X)/DZ)

c Relax constraint for precise energy matches
      if(dabs(e-x).lt.0.03.and.z.gt.1.0) z=1.0

c??  Can this happen?
      if(iscore.NE.0) then
         if(z.gt.6) return
         if(z.gt.1) z=1.5
         stop 9999
c         go to 521
      endif

      IF(DZ.LE.4.0) GO TO 521
      IF(DZ.GE.10.0) GO TO 524
      MEZ=2

c       DZ is 4-10. No score if Z > 2 sigma
      IF(Z.GT.3.0) RETURN
      IF(Z.GT.2.0) GO TO 520
      GO TO 522

c       DZ is >10. No score if Z > 2 sigma
524   IF(Z.GT.2.0) RETURN
      MEZ=4
      GO TO 523

c       DZ is 0-4. No score if Z > 3 sigma
521   CONTINUE
      IF(Z.GT.4.0) RETURN
      IF(Z.GT.3.0) GO TO 520
      IF(Z.LT.3.0) JSCORE=2
522   IF(Z.LT.2.0) JSCORE=5-MEZ
523   IF(Z.LT.1.0) JSCORE=7-MEZ
      IF(Z.LT.0.3) JSCORE=9-MEZ
520   CONTINUE

c     Add energy score to ISCORE and return
      ISCORE=ISCORE+JSCORE

      RETURN
      END

      SUBROUTINE MJSCOR(MULL,MJ1,MJ2,IODD,ISCORE)
C  This subroutine matches multipolarity to initial and final spins
C  Score 0 for partial match, 7 for complete match and -7 for mismatch

      CHARACTER*1 JPAR(5),KPAR(5),IERR
      CHARACTER*2 MEL(15)
      CHARACTER*3 MJP(5)
      CHARACTER*10 MUL,MULL
      CHARACTER*18 MJ1,MJ2,IBLANK
      DIMENSION JSPIN(5),KSPIN(5)

      DATA MEL/'E1','M2','E3','M4','E5','E0','M1','E2','M3','E4','M5',
     2'E6','D ','Q ','O '/

      IBLANK='                  '
c     No score if any of the spin fields is empty
      IF(MJ1.EQ.IBLANK.OR.MJ2.EQ.IBLANK) RETURN

      MUL=MULL
c     Treat no multipolarity as Quadrupole
      IF(MUL.EQ.'          ') MUL='  Q       '

c     No score if multipolarity is based on spins (IF or [])
      IF(INDEX(MUL,'IF').NE.0.OR.INDEX(MUL,'[').NE.0) RETURN
      JT=-1

c     Is multipolarity dipole (D)? This takes care of D+Q
      IF(INDEX(MUL,'D').NE.0) THEN
         L=13
         GO TO 11
      ENDIF

      DO 10 L=1,15
         LL=2
         IF(LL.GT.12) LL=1
         IF(INDEX(MUL,MEL(L)(1:LL)).NE.0) GO TO 11
10    CONTINUE

      JT=0
c     No match found, return
      IF(L.EQ.15) RETURN

11    CONTINUE
      LDQ=0
c     Dipole/quadrupole/octupole
      IF(L.GT.12) LDQ=1
      IF(L.EQ.13) L=1
      IF(L.EQ.14) L=2
      IF(L.EQ.15) L=3

      IF(L.LE.5) GO TO 12
      L=L-6
      JT=1

c     JT=1 for no parity change. JT=0 for parity change. L is multipolarity
12    CONTINUE
c     Double multipolarity to handle half-integer spins
      L=L*IODD
      NMAT=0

C  MJ1 and MJ2 are the inital and final level spins respectively
      CALL JREAD(MJ1,JSPIN,JPAR,MJP,N1,IERR)
      CALL JREAD(MJ2,KSPIN,KPAR,MJP,N2,IERR)

C  PROCESS  D, Q, O
      IF(LDQ.NE.0) THEN
         DO 30 I=1,N1
            DO 31 J=1,N2
               IF(IABS(JSPIN(I)-KSPIN(J)).GT.L) GO TO 31
               NMAT=NMAT+1
31          CONTINUE
30       CONTINUE
      GO TO 25
      ENDIF

C  Assume M2 and higher multipoles are stretched
      DO 20 I=1,N1
         DO 21 J=1,N2
            IF(L.GT.2*IODD.OR.(L.EQ.2*IODD.AND.JT.EQ.-1)) GOTO 22
            IF(JSPIN(I)-L.GT.KSPIN(J).OR.JSPIN(I)+L.LT.KSPIN(J)) GOTO 21
            GO TO 23
22          CONTINUE
            IF(JSPIN(I)-L.NE.KSPIN(J).AND.JSPIN(I)+L.NE.KSPIN(J))
     $       GOTO 21
23          CONTINUE
            IF(JT.EQ.0.OR.JPAR(I).EQ.' '.OR.KPAR(J).EQ.' ')
     $       GO TO 21
            IF(JT.EQ.1.AND.JPAR(I).NE.KPAR(J)) GO TO 21
            IF(JT.EQ.-1.AND.JPAR(I).EQ.KPAR(J)) GO TO 21
            NMAT=NMAT+1
            GO TO 20
21       CONTINUE
20    CONTINUE

C  Score 0 for partial match, 7 for complete match and -7 for mismatch
25    CONTINUE
      IF(NMAT.EQ.N1) ISCORE=ISCORE+7
      IF(NMAT.EQ.0) ISCORE=ISCORE-7
      RETURN
      END

      subroutine SpSn(str,SkipLevel)
      implicit none
      include 'fconv.inc'
      character*9 str
      integer IGET
      logical SkipLevel

      SkipLevel=.False.
      if(index(str,'SP').gt.0) then
c        Make sure Sp for the correct nucleus is used
         if(NucId.NE.SNucId) then
            write(4,100)' *E*',
     $        ' Incorrect Sp. Add Q card. ',
     $        ' Level skipped except for direct ENSDF/ENSDF2 conversion'
     $       ,'Level=',EL(NLEV),'+Sp' 
100         format(/A,A/4X,A/5X,A,F12.3,A)
            SkipLevel=.true.
            return
         endif
         KRatio=IGET(DSId(1:10))/float(A)
         EL(NLEV)=Sp+KRatio*x(1)
         DEL(NLev)=sqrt(DSp**2+x(2)**2)
         Sep=.true.
      elseif(index(str,'SN').gt.0) then
c        Make sure Sn for the correct nucleus is used
         if(NucId.NE.SNucId) then
            write(4,100)' *E*',
     $        ' Incorrect Sn. Add Q card. ',
     $        ' Level skipped except for direct ENSDF/ENSDF2 conversion'
     $       ,'Level=',EL(NLEV),'+Sn' 
            SkipLevel=.true.
            return 
         endif

         KRatio=IGET(DSId(1:10))/float(A)
         EL(NLEV)=Sn+KRatio*x(1)
         DEL(NLev)=sqrt(DSn**2+x(2)**2)
         Sep=.true.
      endif

      return
      end

      SUBROUTINE WBAND(OUT)
c  Writes band data using common block data

      implicit none
      integer OUT,i,ii,j
      character*133 ocard
      logical first

      INCLUDE 'fconv.inc'

      if(NBand.gt.0) then
c        Write nuclide identification
         WRITE(ocard,'('' * '',A5,'' Z='',I3,'' N='',I3,1X,A)')
     $      nucid(1:5),Z,(A-Z),DSId(1:40)
         write(out,*)
         call cardout(out,ocard)
c        Determine band properties
         call BdProp
      endif


      do j=1,NBAND
         first=.true.
c        Write band label,spin string,properties and comment  
         if(iodd.eq.1) then
            write(ocard,101) BD(j),BdSTR(j),BdK(j),BdPi(j),
     $        BdStep(j),BANDID(j)
101         format('     BAND(',A,') JPi: ',A,'[',I3,A,I2,'] ',A)
         else
            write(ocard,102) BD(j),BdSTR(j),BdK(j),BdPi(j),
     $        BdStep(j)/2,BANDID(j)
102         format('     BAND(',A,') JPi: ',A,'[',I3,'/2',A,I2,'] ',A)
         endif
         call cardout(out,ocard)
c        Write levels
         do i=1,NLev
c           Prepare level card
            ii=loo(i)
            if(ii.le.0) goto 10
            if(Ban(ii).eq.BD(j)) then
c              Write table headers   
               if(first) then
                  ocard=' '
                  ocard(11:12)='Ex'
                  ocard(19:22)='Spin' 
                  if(mel(ii).ne.' ') ocard(13:14)='-'//mel(ii)
                  if(JOffs(ii).ne.' ') ocard(23:24)='-'//JOffs(ii)
                  call cardout(out,ocard)
               endif
               ocard=' '
c              Level energy
               write(ocard(5:14),'(F10.2)') el(ii)
               ocard(15:15)=UncLev(ii)
c              JPi
               if(spin(ii).ge.0)
     $          write(ocard(18:22),'(F5.1)') spin(ii)/float(iodd)
c              Write level card
               call cardout(out,ocard)
               first=.false.
            endif
10       continue
         enddo

      enddo

      return
      END
      subroutine Skip(INP)
c
c     Skips cards until EOF or End of Data Set
c
      INCLUDE 'fconv.inc'

10    read(INP,'(A)',end=99) card(1)
      if(card(1)(1:10).ne.' ') goto 10
      return

c     End-of-file found
99    TheEnd=.TRUE.

      return
      end

C=================================================================
      
c      SUBROUTINE WAGS(OUT,mode)
c  Writes AGS file using common block data

C=================================================================

      subroutine mul(ll,dum2)
c
c     Get multipolarity of gamma ll for WAGS.
c
      implicit none

      include 'fconv.inc'

      character*(*) dum2

      integer ll

      dum2='  '

c     Get multipolarity from ML and MLPar
      if(ML(1,ll).ge.0) then
         write(dum2(2:2),'(I1)') ML(1,ll)
         if(MLPar(ll).ne.' ') then
            dum2(1:1)='E'
            if(MLPar(ll).eq.'+') then
               if((-1)**ML(1,ll).eq.-1) dum2(1:1)='M'
            elseif(MLPar(ll).eq.'-') then
               if((-1)**ML(1,ll).eq.1) dum2(1:1)='M'
            endif
         endif
      endif

      return
      end

      subroutine label(ll,iband,n,name)
c
c     Get name for level ll for WAGS.
c     ll is level number, n is level number in band, name is
c     the generated label
c
      implicit none

      include 'fconv.inc'

      character*(*) name,dum*5

      integer ll,ispin,n,iband

      dum=Nucid
      call tbsup(dum)
      call padlft(dum,5)
      name=dum

      if(Ban(ll).ne.' ') then
c        Band level, band number iband, level number n
         name(6:6)=BdPi(iband)
         name(7:7)=Bd(iband)
         if(spin(ll).ge.0) then
c           Known spin
            if(JOffs(ll).eq.' ') then
c              No J offset
               ispin=spin(ll)/iodd
               write(name(9:10),104) ispin
104            format(I2.2)
            else
c              With J offset
               ispin=(BdK(iband)+(n-1)*BdStep(iband))/iodd
               if(ispin.ge.0) then
                  write(name(9:10),104) ispin
                  if(ispin.eq.0.and.iodd.eq.2) name(9:10)='-0'  
               else
                  name(9:10)='-1'
               endif
            endif 
         else
c           Unknown spin
            ispin=(BdK(iband)+(n-1)*BdStep(iband))/iodd
            if(ispin.ge.0) then
               write(name(9:10),104) ispin
               if(ispin.eq.0.and.iodd.eq.2) name(9:10)='-0'  
            else
               name(9:10)='-1'
            endif
         endif
         name(8:8)='O'
         if((ispin/2)*2.eq.ispin) name(8:8)='E'
      else
c        Non-band level, level number 0
         name(6:6)=Parity(ll)
         if(name(6:6).eq.' ') name(6:6)='?'
         call charad(ll+151,name(7:8))
         ispin=spin(ll)/iodd
         if(ispin.ge.0) then
            write(name(9:10),104) ispin
            if(ispin.eq.0.and.iodd.eq.2) name(9:10)='-0'  
         else
            name(9:10)='-1'
         endif
      endif

      return
      end

      subroutine rpar(line,lline,pstr,lpstr,npp)
      implicit none

      character*(*) line,pstr(20)
      integer npp,lpstr(20),lline,posb,pose,i

      npp=0
      posb=1

1     continue
c     Find first non-space character
      do i=posb,lline
         if(line(i:i).ne.' ') goto 10
      enddo
      goto 99

10    continue
      posb=i
      npp=npp+1

c     Find first space
      do i=posb,lline
         if(line(i:i).eq.' ') goto 20
      enddo
      i=lline+1

20    continue
      pose=i-1
      lpstr(npp)=pose-posb+1
      pstr(npp)=line(posb:pose)
      line(posb:pose)=' '
      posb=pose+1
      goto 1

99    continue
      return
      end

      subroutine sort(N,lo,loo,a)

c     Sort the N entries a in ascending order. The order is defined
c     in the array lo. lo is 1 for the first level 2 for the second, etc.
c     Method: count number of values of a that are smaller or equal
c     than the current current. The array loo is the inverse array
c     to be used when picking levels in energy order.
c
c     Example:
c
c       i     lo(i)    loo(i)    a(i)
c       ------------------------------
c       1       1        1       100
c       2       3        3       300
c       3       2        2       200
c       4       4        4       400
c       5       7        6       700
c       6       5        7       500
c       7       6        5       600
c       8       8        8       800
c

      implicit none

      integer N,lo(*),loo(*),i,j,count

      double precision a(*)

      do i=1,N
         count=0
         do j=1,N
            if(a(j).le.a(i)) count=count+1
c           Make sure that not two level energies are the same
            if(i.ne.j.and.a(j).eq.a(i)) then
               write(4,'(/'' *E* Two levels with same level energy:'',
     $         F10.3)') a(j)
            endif
         enddo
         lo(i)=count
      enddo

      do j=1,N
         loo(j)=0
         do i=1,N
            if(lo(i).eq.j) then
               loo(j)=i
               goto 10
            endif
         enddo
10       continue
      enddo

      return
      end
      subroutine nextfile(listfile,name,ext,outname,outext)
      implicit none

      character*80 instr
      character name*(*),outname*(*),ext*3,outext*3
      integer listfile,lext

      name=' '

1     read(listfile,100,END=99) instr
100   format(a)
c  Ignore input not starting in column 1
      if(instr(1:1).eq.' ') goto 1
c  Find out if there is a .
      lext=index(instr(1:13),'.')

      if(lext.gt.0) then
c  VMS dir
c  Check extension
         if(ext.eq.' ') then
            ext=instr(lext+1:lext+3)
         elseif(ext.ne.instr(lext+1:lext+3)) then
            write(*,*) ' Invalid file extension=',instr(lext+1:lext+3)
            stop
         endif
      else
c  DOS dir
c  Check extension
         lext=9
         if(ext.eq.' ') then
            ext=instr(10:12)
         elseif(ext.ne.instr(10:12)) then
            write(*,*) ' Invalid file extension=',instr(10:12)
            stop
         endif
      endif
      name=instr(1:lext-1)
      lext=index(name,' ')
      name=name(1:lext-1)//'.'//ext
      outname=name(1:lext-1)//'.'//outext

99    return
      end
      subroutine LevStat(UncLev,Ispin,parity,JPiMod,JOffs,itent)

c     Determines level uncertainty status variable (itent) for AGS format
c     from UncLev and JPiMod:
c
c      itent     Label in Radware    JPiMod    Ispin  parity
c       0         J Pi                '   '     ge 0   ne ' '
c       1         (J) Pi              ' (J'     ge 0   ne ' '
c       2         J (Pi)              ' (P'     ge 0   ne ' '
c       3         (J Pi)              ' (B'     ge 0   ne ' '
c       4         J                   '   '     ge 0   ' '
c       5         (J)                 ' (J'     ge 0   ' '
c       6         none                any       lt 0   any
c  JPffs ne ' ' gives label 6 
    
      implicit none
      character*1 UncLev,JPiMod*3,parity,JOffs
      integer itent,Ispin

      itent=6
      if(JOffs.ne.' ') then
         itent=6
      elseif(Ispin.lt.0) then
         itent=6
      elseif(Ispin.ge.0.and.parity.eq.' '.and.JPiMod(3:3).eq.'J') then
         itent=5
      elseif(Ispin.ge.0.and.parity.eq.' '.and.JPiMod(3:3).eq.' ') then
         itent=4
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.'B') then
         itent=3
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.'P') then
         itent=2
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.'J') then
         itent=1
      elseif(Ispin.ge.0.and.parity.ne.' '.and.JPiMod(3:3).eq.' ') then
         itent=0
      endif

      return
      end

      subroutine BdProp

c     Determinies band properties (parity, spin step) for a band.
c     The parameters are mainly used for the naming of levels by
c     WAGS.

      implicit none
      integer i,j,ii,counter,lastspin,idiff,iget2
      logical BsOK
      INCLUDE 'fconv.inc'
    
      do j=1,NBand
c        Define default band properties
         BsOK=.false.
         BdPi(j)='?'
         BdK(j)=-1
         BdStep(j)=2*iodd
         counter=0
         lastspin=-99

         do i=1,NLev
            ii=loo(i)
            if(ii.le.0) goto 10
            if(Ban(ii).eq.BD(j)) then
c              Parity of band determined by first parity given
               if(parity(ii).ne.' '.and.BdPi(j).eq.'?') then
                  BdPi(j)=parity(ii)
               endif 
c              BdStep is 1 or 2. Determined by first spin difference.
c              BdK is calculated from this or from spin of first level
               counter=counter+1
               if(counter.eq.1) then
                  ioffs(j)=0
c                 Get J offset if given
                  if(JOffs(ii).ne.' ') then
                     ioffs(j)=iget2(BdStr(j))
                     if(ioffs(j).le.0) ioffs(j)=-1
                  endif
                  if(spin(ii).ge.0) then
                     BdK(j)=spin(ii)+ioffs(j)
                  else
                     BdK(j)=spin(ii)
                  endif
               endif
               if(counter.ge.2) then
                  idiff=(spin(ii)-lastspin)
                  if((idiff/iodd.eq.1.or.idiff/iodd.eq.2)
     $                                           .and..not.BsOK) then
                     BdStep(j)=idiff
                     if(iodd.eq.2.and.JOffs(ii).ne.' ') then
                        BdStep(j)=BdStep(j)*2
                     endif
                     if(BdK(j).lt.0.and.JOffs(ii).eq.' ')
     $                 BdK(j)=spin(ii)-(counter-1)*BdStep(j)
                     BsOK=.true.
                  endif
               endif
               if(spin(ii).ge.0) lastspin=spin(ii)
            endif 
         enddo
10       continue
      enddo
      return
      end

      subroutine bandcheck
c     Check for duplicate labels
      implicit none
      integer i,j
      INCLUDE 'fconv.inc'
c  Check labels
      do i=1,NLev-1
         do j=i+1,NLev
            if(LevLab(i).eq.LevLab(j)) then
               write(4,100)' *E* Duplicate label= ',LevLab(i),
     $           'Levels= ',EL(i),EL(j) 
100            format(/A,A/5X,A,2F12.3)
            endif
         enddo
      enddo

c  Check parities
      do j=1,NBAND
         do i=1,NLev
            if(Ban(i).eq.BD(j).and.BdPi(j).ne.'?') then
               if(parity(i).ne.' '.and.parity(i).ne.BdPi(j)) then
                  write(4,101)' *E* Parity deviates from band parity',
     $              'Level: ',EL(i),' Parities: ',parity(i),BdPi(j)
101               format(/A/5X,A,F10.2,A,A,1X,A)
               endif
            endif
         enddo
      enddo

      return
      end

      subroutine commonBlocks(mode,pos)
c     Dump/reset common blocks 
      implicit none
	character pos*(*)
	integer mode,i,j

      INCLUDE 'fconv.inc'
	  
	if(mode.eq.-1.or.mode.eq.2) then
c      Reset variables  
		idum=0
		do i=1,3 
			x(i)=0
		enddo
		dumstr=' '

		A=0
		Z=0
		IODD=0
		SP=0
		DSP=0
		SN=0
		DSN=0
		KRatio=1
		Sep=.false.
		LComm=0
		NLev=0
	    do j=1,MLev
			EL(j)=0
			DEL(j)=0
			Iscore(j)=0
			L(j)=0
			LO(j)=j
			NDec(j)=0
			Spin(j)=0
			T(j)=0
			DT(j)=0
			DT2(j)=0
			LOO(j)=j	
		enddo
	    do j=1,MBand
			BdStep(j)=0
			BdK(j)=0
			ioffs(j)=0
		enddo
		do j=1,MGam
			NGam=0
			EGam(j)=0
			DEGam(j)=0
			IGam(j)=0
			DIGam(j)=0
			InLev(j)=0
			FinLev(j)=0
			D(j)=0
			DD(j)=0
			DD2(j)=0
			ML(1,j)=0
			ML(2,j)=0
		enddo


	  endif

	  if(mode.eq.1.or.mode.eq.2) then
c        Dump variables
		 idum=3
	     write(4,*) 
		 write(4,*)' Common block dump, position=', pos
	     write(4,*)' /fconv/: ', logon
	     write(4,*)' /dum1/: ', idum,x
		 write(4,*)' /dums/: ',dumstr
 	     write(4,*)' /dat/: ', sdate,' ',stime
	     write(4,*)' /dset/: ',A,Z,IODD,SP,DSP,SN,DSN,KRatio
	     write(4,*) Sep,DSStrL,LComm

	     write(4,*)' /levels/: MLev=',MLev,' MBand=',MBand
	     write(4,*)' NLev=',NLev
	     write(4,*)' EL,DEL=',(EL(j),DEL(j),j=1,idum)
		 write(4,*)' Iscore=',(Iscore(j),j=1,idum)
	     write(4,*)' L=',(L(j),j=1,idum)
	     write(4,*)' LO=',(LO(j),j=1,idum)
	     write(4,*)' NDec=',(NDec(j),j=1,idum)
	     write(4,*)' Spin=',(Spin(j),j=1,idum)
	     write(4,*)' T,DT,DT2=',(T(j),DT(j),DT2(j),j=1,idum)
	     write(4,*)' LOO=',(LOO(j),j=1,idum)
	     write(4,*)' BdStep=',(BdStep(j),j=1,idum)
	     write(4,*)' BdK=',(BdK(j),j=1,idum)
	     write(4,*)' ioffs=',(ioffs(j),j=1,idum)

		 write(4,*)' /gammas/: MGam=',MGam
	     write(4,*)' NGam=',NGam
	     write(4,*)' EGam,DEGam=',(EGam(j),DEGam(j),j=1,idum)
	     write(4,*)' IGam,DIGam=',(IGam(j),DIGam(j),j=1,idum)
	     write(4,*)' InLev,FinLev=',(InLev(j),FinLev(j),j=1,idum)
	     write(4,*)' D,DD,DD2=',(D(j),DD(j),DD2(j),j=1,idum)
	     write(4,*)' ML(1),ML(2)=',(ML(1,j),ML(2,j),j=1,idum)

	     write(4,*)


	  endif


      return
      end
